@@ -1,4 +1,6 @@
 #pylint: disable=W0611
+#pylint: disable=C0414
+
 # We import the names "as" themselves here (and here only) to tell linting tools
 # that they are explicitly being exported here (and not just unused).
 
@@ -6,7 +8,6 @@ from static_frame.core.batch import Batch as Batch
 from static_frame.core.bus import Bus as Bus
 from static_frame.core.display import Display as Display
 from static_frame.core.display import DisplayActive as DisplayActive
-from static_frame.core.interface_meta import InterfaceMeta as InterfaceMeta
 from static_frame.core.display_config import DisplayConfig as DisplayConfig
 from static_frame.core.display_config import DisplayConfigs as DisplayConfigs
 from static_frame.core.display_config import DisplayFormats as DisplayFormats
@@ -26,6 +27,7 @@ from static_frame.core.exception import StoreFileMutation
 from static_frame.core.frame import Frame as Frame
 from static_frame.core.frame import FrameAssign as FrameAssign
 from static_frame.core.frame import FrameGO as FrameGO
+from static_frame.core.frame import FrameHE as FrameHE
 from static_frame.core.hloc import HLoc as HLoc
 from static_frame.core.index import ILoc as ILoc
 from static_frame.core.index import Index as Index
@@ -55,6 +57,7 @@ from static_frame.core.index_hierarchy import IndexHierarchy as IndexHierarchy
 from static_frame.core.index_hierarchy import IndexHierarchyGO as IndexHierarchyGO
 from static_frame.core.index_level import IndexLevel as IndexLevel
 from static_frame.core.index_level import IndexLevelGO as IndexLevelGO
+from static_frame.core.interface_meta import InterfaceMeta as InterfaceMeta
 from static_frame.core.node_dt import InterfaceDatetime
 from static_frame.core.node_iter import IterNodeApplyType as IterNodeApplyType
 from static_frame.core.node_iter import IterNodeAxis
@@ -73,9 +76,12 @@ from static_frame.core.node_selector import InterfaceGetItem as InterfaceGetItem
 from static_frame.core.node_selector import InterfaceSelectDuo
 from static_frame.core.node_selector import InterfaceSelectQuartet
 from static_frame.core.node_selector import InterfaceSelectTrio
+from static_frame.core.node_transpose import InterfaceTranspose
 from static_frame.core.node_str import InterfaceString
+from static_frame.core.quilt import Quilt as Quilt
 from static_frame.core.series import Series as Series
 from static_frame.core.series import SeriesAssign as SeriesAssign
+from static_frame.core.series import SeriesHE as SeriesHE
 from static_frame.core.store import StoreConfig as StoreConfig
 from static_frame.core.store import StoreConfigMap as StoreConfigMap
 from static_frame.core.store_filter import StoreFilter as StoreFilter
@@ -92,5 +98,6 @@ from static_frame.core.util import mloc as mloc
 from static_frame.core.util import PathSpecifierOrFileLike as PathSpecifierOrFileLike
 from static_frame.core.util import SeriesInitializer as SeriesInitializer
 
-__version__ = '0.6.35' # use -dev for new version in development
+__version__ = '0.8.0' # use -dev for new version in development
+
 
