@@ -23,14 +23,14 @@ from static_frame.core.util import DTYPE_FLOAT_DEFAULT
 from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import NameType
-from static_frame.core.util import NULL_SLICE
+from static_frame.core.util import INT_TYPES
 
 #-------------------------------------------------------------------------------
-class FrameDefferedMeta(type):
+class FrameDeferredMeta(type):
     def __repr__(cls) -> str:
         return f'<{cls.__name__}>'
 
-class FrameDeferred(metaclass=FrameDefferedMeta):
+class FrameDeferred(metaclass=FrameDeferredMeta):
     '''
     Token placeholder for :obj:`Frame` not yet loaded.
     '''
@@ -38,7 +38,7 @@ class FrameDeferred(metaclass=FrameDefferedMeta):
 #-------------------------------------------------------------------------------
 class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     '''
-    A lazy, randomly-accessible container of :obj:`Frame`.
+    A randomly-accessible container of :obj:`Frame`. When created from a multi-table storage format (such as a zip-pickle or XLSX), a Bus will lazily read in components as they are accessed. When combined with the ``max_persist`` parameter, a Bus will not hold on to more than ``max_persist`` references, permitting low-memory reading of collections of :obj:`Frame`.
     '''
 
     __slots__ = (
@@ -47,20 +47,24 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         '_series',
         '_store',
         '_config',
+        '_last_accessed',
+        '_max_persist',
         )
 
     _series: Series
     _store: tp.Optional[Store]
     _config: StoreConfigMap
+    _name: NameType
 
     STATIC = False
 
     @staticmethod
-    def _deferred_series(labels: tp.Iterable[str]) -> Series:
+    def _deferred_series(labels: tp.Iterable[tp.Hashable]) -> Series:
         '''
         Return an object ``Series`` of ``FrameDeferred`` objects, based on the passed in ``labels``.
         '''
-        return Series.from_element(FrameDeferred, index=labels, dtype=object)
+        # NOTE: need to accept an  IndexConstructor to support reanimating Index subtypes, IH
+        return Series.from_element(FrameDeferred, index=labels, dtype=DTYPE_OBJECT)
 
     @classmethod
     def from_frames(cls,
@@ -71,22 +75,36 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             ) -> 'Bus':
         '''Return a :obj:`Bus` from an iterable of :obj:`Frame`; labels will be drawn from :obj:`Frame.name`.
         '''
-        # could take a StoreConfigMap
         series = Series.from_items(
                     ((f.name, f) for f in frames),
-                    dtype=object,
+                    dtype=DTYPE_OBJECT,
                     name=name,
                     )
         return cls(series, config=config)
 
+    @classmethod
+    def from_items(cls,
+            pairs: tp.Iterable[tp.Tuple[tp.Hashable, Frame]],
+            *,
+            config: StoreConfigMapInitializer = None,
+            name: NameType = None,
+            ) -> 'Bus':
+        '''Return a :obj:`Bus` from an iterable of pairs of label, :obj:`Frame`.
+        '''
+        series = Series.from_items(pairs, dtype=DTYPE_OBJECT, name=name)
+        return cls(series, config=config)
+
     @classmethod
     def _from_store(cls,
             store: Store,
-            config: StoreConfigMapInitializer = None
+            *,
+            config: StoreConfigMapInitializer = None,
+            **kwargs: tp.Any,
             ) -> 'Bus':
-        return cls(cls._deferred_series(store.labels()),
+        return cls(cls._deferred_series(store.labels(config=config)),
                 store=store,
-                config=config
+                config=config,
+                **kwargs,
                 )
 
     #---------------------------------------------------------------------------
@@ -94,24 +112,29 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             series: Series,
             *,
             store: tp.Optional[Store] = None,
-            config: StoreConfigMapInitializer = None
+            config: StoreConfigMapInitializer = None,
+            max_persist: tp.Optional[int] = None,
             ):
         '''
         Args:
-            config: StoreConfig for handling ``Frame`` construction and exporting from Store.
+            config: StoreConfig for handling :obj:`Frame` construction and exporting from Store.
+            max_persist: When loading :obj:`Frame` from a :obj:`Store`, optionally define the maximum number of :obj:`Frame` to remain in the :obj:`Bus`, regardless of the size of the :obj:`Bus`. If more than ``max_persist`` number of :obj:`Frame` are loaded, least-recently loaded :obj:`Frame` will be replaced by ``FrameDeferred``. A ``max_persist`` of 1, for example, permits reading one :obj:`Frame` at a time without ever holding in memory more than 1 :obj:`Frame`.
         '''
 
         if series.dtype != DTYPE_OBJECT:
             raise ErrorInitBus(
                     f'Series passed to initializer must have dtype object, not {series.dtype}')
 
+        if max_persist is not None:
+            # use an (ordered) dictionary to give use an ordered set, simply pointing to None for all keys
+            self._last_accessed: tp.Dict[str, None] = {}
+
         # do a one time iteration of series
         def gen() -> tp.Iterator[bool]:
             for label, value in series.items():
-                if not isinstance(label, str):
-                    raise ErrorInitBus(f'supplied label {label} is not a string.')
-
                 if isinstance(value, Frame):
+                    if max_persist is not None:
+                        self._last_accessed[label] = None
                     yield True
                 elif value is FrameDeferred:
                     yield False
@@ -123,18 +146,23 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         self._series = series
         self._store = store
 
+        # Not handling cases of max_persist being greater than the length of the Series (might floor to length)
+        if max_persist is not None and max_persist < self._loaded.sum():
+            raise ErrorInitBus('max_persis cannot be less than the number of already loaded Frames')
+        self._max_persist = max_persist
+
         # providing None will result in default; providing a StoreConfig or StoreConfigMap will return an appropriate map
         self._config = StoreConfigMap.from_initializer(config)
 
     #---------------------------------------------------------------------------
     # delegation
 
-    def __getattr__(self, name: str) -> tp.Any:
+    def __getattr__(self, attr: str) -> tp.Any:
         try:
-            return getattr(self._series, name)
+            return getattr(self._series, attr)
         except AttributeError:
             # fix the attribute error to reference the Bus
-            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")
+            raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{attr}'") from None
 
     #---------------------------------------------------------------------------
     # cache management
@@ -149,40 +177,116 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             return [self.index.values[key],] # needs to be a list for usage in loc assignment
         return self.index.values[key]
 
+    @staticmethod
+    def _store_reader(
+            store: Store,
+            config: StoreConfigMap,
+            labels: tp.Iterator[tp.Hashable],
+            max_persist: tp.Optional[int],
+            ) -> tp.Iterator[Frame]:
+        '''
+        Ready as many labels as possible from Store, then yield back each one at a time. If max_persist is active, max_persist will set the maximum number of Frame to load per read. Using Store.read_many is shown to have significant performance benefits on large collections of Frame.
+        '''
+        if max_persist is None:
+            for frame in store.read_many(labels, config=config):
+                yield frame
+        elif max_persist > 1:
+            coll = []
+            for label in labels:
+                coll.append(label)
+                if len(coll) == max_persist:
+                    for frame in store.read_many(coll, config=config):
+                        yield frame
+                    coll.clear()
+            if coll: # less than max persist remaining
+                for frame in store.read_many(coll, config=config):
+                    yield frame
+        else: # max persist is 1
+            for label in labels:
+                yield store.read(label, config=config[labels])
+
 
     def _update_series_cache_iloc(self, key: GetItemKeyType) -> None:
         '''
         Update the Series cache with the key specified, where key can be any iloc GetItemKeyType.
+
+        Args:
+            key: always an iloc key.
         '''
+        max_persist_active = self._max_persist is not None
 
-        # do nothing if all loaded, or if the requested keys are already loadsed
-        if not self._loaded_all and not self._loaded[key].all():
-            if self._store is None:
-                raise RuntimeError('no store defined')
+        load = False if self._loaded_all else not self._loaded[key].all()
+        if not load and not max_persist_active:
+            return
 
-            labels = set(self._iloc_to_labels(key))
+        index = self._series.index
+        if not load and max_persist_active: # must update LRU position
+            labels = (index.iloc[key],) if isinstance(key, INT_TYPES) else index.iloc[key].values
+            for label in labels: # update LRU position
+                self._last_accessed[label] = self._last_accessed.pop(label, None)
+            return
 
-            array = np.empty(shape=len(self._series._index), dtype=object)
-            for idx, (label, frame) in enumerate(self._series.items()):
-                if frame is FrameDeferred and label in labels:
-                    frame = self._store.read(label, config=self._config[label])
-                    self._loaded[idx] = True # update loaded status
+        if self._store is None: # there has to be a Store defined if we are partially loaded
+            raise RuntimeError('no store defined')
+        if max_persist_active:
+            loaded_count = self._loaded.sum()
+
+        array = self._series.values.copy() # not a deepcopy
+        targets = self._series.iloc[key] # key is iloc key
+
+        if not isinstance(targets, Series):
+            label = index[key] #type: ignore [unreachable]
+            targets_items = ((label, targets),) # present element as items
+            store_reader = (self._store.read(label, config=self._config[label]) for _ in  range(1))
+        else: # more than one Frame
+            store_reader = self._store_reader(
+                    store=self._store,
+                    config=self._config,
+                    labels=(label for label, f in targets.items() if f is FrameDeferred),
+                    max_persist=self._max_persist,
+                    )
+            targets_items = targets.items()
+
+        for label, frame in targets_items:
+            idx = index.loc_to_iloc(label)
+
+            if max_persist_active: # update LRU position
+                self._last_accessed[label] = self._last_accessed.pop(label, None)
+
+            if frame is FrameDeferred:
+                frame = next(store_reader)
+
+            if not self._loaded[idx]:
+                # as we are iterating from `targets`, we might be holding on to references of Frames that we already removed in `array`; in this case we do not need to `read`, but we still need to update the new array
                 array[idx] = frame
-            array.flags.writeable = False
+                self._loaded[idx] = True # update loaded status
+                if max_persist_active:
+                    loaded_count += 1
 
-            self._series = Series(array, index=self._series._index, dtype=object)
-            self._loaded_all = self._loaded.all()
+            if max_persist_active and loaded_count > self._max_persist:
+                label_remove = next(iter(self._last_accessed))
+                del self._last_accessed[label_remove]
+                idx_remove = index.loc_to_iloc(label_remove)
+                self._loaded[idx_remove] = False
+                array[idx_remove] = FrameDeferred
+                loaded_count -= 1
 
-    def _update_series_cache_all(self) -> None:
-        '''Load all Tables contained in this Bus.
-        '''
-        if not self._loaded_all:
-            self._update_series_cache_iloc(NULL_SLICE)
+        array.flags.writeable = False
+        self._series = Series(array,
+                index=self._series._index,
+                dtype=object,
+                own_index=True,
+                )
+        self._loaded_all = self._loaded.all()
 
     #---------------------------------------------------------------------------
     # extraction
 
     def _extract_iloc(self, key: GetItemKeyType) -> 'Bus':
+        '''
+        Returns:
+            Bus or, if an element is selected, a Frame
+        '''
         self._update_series_cache_iloc(key=key)
 
         # iterable selection should be handled by NP
@@ -190,6 +294,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
 
         if not isinstance(values, np.ndarray): # if we have a single element
             return values #type: ignore
+
         series = Series(
                 values,
                 index=self._series._index.iloc[key],
@@ -197,6 +302,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         return self.__class__(series=series,
                 store=self._store,
                 config=self._config,
+                max_persist=self._max_persist,
                 )
 
     def _extract_loc(self, key: GetItemKeyType) -> 'Bus':
@@ -209,22 +315,19 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         values = self._series.values[iloc_key]
 
         if not isinstance(values, np.ndarray): # if we have a single element
-            # NOTE: only support str labels, not IndexHierarchy
-            # if isinstance(key, HLoc) and key.has_key_multiple():
-            #     values = np.array(values)
-            #     values.flags.writeable = False
             return values #type: ignore
 
         series = Series(values,
                 index=self._series._index.iloc[iloc_key],
                 own_index=True,
                 name=self._series._name)
+
         return self.__class__(series=series,
                 store=self._store,
                 config=self._config,
+                max_persist=self._max_persist,
                 )
 
-
     @doc_inject(selector='selector')
     def __getitem__(self, key: GetItemKeyType) -> 'Bus':
         '''Selector of values by label.
@@ -253,20 +356,26 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         return self._series.__len__()
 
     #---------------------------------------------------------------------------
-    # dictionary-like interface
+    # dictionary-like interface; these will force loadings contained Frame
 
-    def items(self) -> tp.Iterator[tp.Tuple[str, tp.Any]]:
-        '''Iterator of pairs of index label and value.
+    def items(self) -> tp.Iterator[tp.Tuple[tp.Hashable, Frame]]:
+        '''Iterator of pairs of :obj:`Bus` label and contained :obj:`Frame`.
         '''
-        self._update_series_cache_all()
-        yield from self._series.items()
+        # force new iteration to account for max_persist
+        for i, label in enumerate(self._series._index):
+            yield label, self._extract_iloc(i) #type: ignore
+
+    _items_store = items
 
     @property
     def values(self) -> np.ndarray:
-        '''A 1D array of values.
+        '''A 1D object array of all Frame contained in the Bus.
         '''
-        self._update_series_cache_all()
-        return self._series.values
+        post = np.empty(self.__len__(), dtype=object)
+        for i, _ in enumerate(self._series._index):
+            post[i] = self._extract_iloc(i)
+
+        return post
 
     #---------------------------------------------------------------------------
     @doc_inject()
@@ -286,7 +395,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         return self._series._display(config, display_cls)
 
     #---------------------------------------------------------------------------
-    # extended disciptors
+    # extended discriptors; in general, these do not force loading Frame
 
     @property
     def mloc(self) -> Series:
@@ -319,7 +428,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
 
     @property
     def shapes(self) -> Series:
-        '''A :obj:`Series` describing the shape of each loaded :obj:`Frame`.
+        '''A :obj:`Series` describing the shape of each loaded :obj:`Frame`. Unloaded :obj:`Frame` will have a shape of None.
 
         Returns:
             :obj:`tp.Tuple[int]`
@@ -327,7 +436,6 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         values = (f.shape if f is not FrameDeferred else None for f in self._series.values)
         return Series(values, index=self._series._index, dtype=object, name='shape')
 
-
     @property
     def nbytes(self) -> int:
         '''Total bytes of data currently loaded in the Bus.
@@ -337,7 +445,7 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
     @property
     def status(self) -> Frame:
         '''
-        Return a
+        Return a :obj:`Frame` indicating loaded status, size, bytes, and shape of all loaded :obj:`Frame`.
         '''
         def gen() -> tp.Iterator[Series]:
 
@@ -371,6 +479,8 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         '''
         {doc}
 
+        Note: this will attempt to load and compare all Frame managed by the Bus.
+
         Args:
             {compare_name}
             {compare_dtype}
@@ -386,17 +496,13 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
         elif not isinstance(other, Bus):
             return False
 
-        # defer updating cache
-        self._update_series_cache_all()
-
+        # NOTE: dtype self._series is always object
         if len(self._series) != len(other._series):
             return False
 
         if compare_name and self._series._name != other._series._name:
             return False
 
-        # NOTE: dtype self._series is always object
-
         if not self._series.index.equals(
                 other._series.index,
                 compare_name=compare_name,
@@ -407,8 +513,8 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
             return False
 
         # can zip because length of Series already match
-        for (frame_self, frame_other) in zip(
-                self._series.values, other._series.values):
+        # using .values will force loading all Frame into memory; better to use items() to permit collection
+        for (_, frame_self), (_, frame_other) in zip(self.items(), other.items()):
             if not frame_self.equals(frame_other,
                     compare_name=compare_name,
                     compare_dtype=compare_dtype,
@@ -418,3 +524,32 @@ class Bus(ContainerBase, StoreClientMixin): # not a ContainerOperand
                 return False
 
         return True
+
+    #---------------------------------------------------------------------------
+    # transformations resulting in changed dimensionality
+
+    @doc_inject(selector='head', class_name='Bus')
+    def head(self, count: int = 5) -> 'Bus':
+        '''{doc}
+
+        Args:
+            {count}
+
+        Returns:
+            :obj:`Bus`
+        '''
+        return self.iloc[:count]
+
+    @doc_inject(selector='tail', class_name='Bus')
+    def tail(self, count: int = 5) -> 'Bus':
+        '''{doc}s
+
+        Args:
+            {count}
+
+        Returns:
+            :obj:`Bus`
+        '''
+        return self.iloc[-count:]
+
+
