@@ -37,16 +37,19 @@ from static_frame.core.util import UFunc
 from static_frame.core.util import ufunc_set_iter
 from static_frame.core.util import INT_TYPES
 from static_frame.core.util import NameType
+from static_frame.core.util import is_dtype_specifier
+from static_frame.core.util import is_mapping
 
 from static_frame.core.exception import AxisInvalid
 
 if tp.TYPE_CHECKING:
     import pandas as pd #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.type_blocks import TypeBlocks #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.series import Series #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.frame import Frame #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.index_auto import IndexAutoFactoryType #pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.type_blocks import TypeBlocks #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.series import Series #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.frame import Frame #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_auto import IndexAutoFactoryType #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.quilt import Quilt #pylint: disable=W0611,C0412 #pragma: no cover
 
 
 
@@ -62,28 +65,34 @@ def get_col_dtype_factory(
     '''
     from static_frame.core.series import Series
 
-    # dtypes are either mappable by name, or an ordered sequence; it might be possible to support a single dtype initializer applied to all columns, however, the types of dtype initialzers are so broad, it is hard to distinguish them from a list (i.e., str class).
-
+    # dtypes are either a dtype initializer, mappable by name, or an ordered sequence
     # NOTE: might verify that all keys in dtypes are in columns, though that might be slow
 
-    if isinstance(dtypes, (dict, Series)):
+    if is_mapping(dtypes):
         is_map = True
-    else:
+        is_element = False
+    elif is_dtype_specifier(dtypes):
         is_map = False
+        is_element = True
+    else: # an iterable of types
+        is_map = False
+        is_element = False
 
     if columns is None and is_map:
         raise RuntimeError('cannot lookup dtypes by name without supplied columns labels')
 
     def get_col_dtype(col_idx: int) -> DtypeSpecifier:
-        nonlocal dtypes
+        nonlocal dtypes # might mutate a generator into a tuple
+        if is_element:
+            return dtypes
         if is_map:
             return dtypes.get(columns[col_idx], None) #type: ignore
-
-        # NOTE: dtypes might be a generator deferred until this function is called; if so, realize here
-        if not hasattr(dtypes, '__len__'):
+        # NOTE: dtypes might be a generator deferred until this function is called; if so, realize here; INVALID_ITERABLE_FOR_ARRAY (dict_values, etc) do not have __getitem__,
+        if not hasattr(dtypes, '__len__') or not hasattr(dtypes, '__getitem__'):
             dtypes = tuple(dtypes) #type: ignore
         return dtypes[col_idx] #type: ignore
 
+
     return get_col_dtype
 
 
@@ -97,17 +106,6 @@ def is_static(value: IndexConstructor) -> bool:
     return getattr(value.__self__, STATIC_ATTR) #type: ignore
 
 
-# def is_dataclass(value: tp.Any) -> bool:
-#     '''Determine if dataclasses are available. Might be installed via a pip package.
-#     '''
-#     # introduced in Python 3.7; remove special handling when min req is 3.7
-#     try:
-#         import dataclasses
-#     except ImportError:
-#         return False
-#     return dataclasses.is_dataclass(value)
-
-
 def pandas_version_under_1() -> bool:
     import pandas
     return not hasattr(pandas, 'NA') # object introduced in 1.0
@@ -193,7 +191,6 @@ def pandas_to_numpy(
     return array
 
 
-
 def index_from_optional_constructor(
         value: IndexInitializer,
         *,
@@ -224,7 +221,7 @@ def index_from_optional_constructor(
             # v: S, dc: ~S, return a mutable version of something that is not mutable
             return value._MUTABLE_CONSTRUCTOR(value)
 
-    # cannot always deterine satic status from constructors; fallback on using default constructor
+    # cannot always determine static status from constructors; fallback on using default constructor
     return default_constructor(value)
 
 def index_constructor_empty(
@@ -244,8 +241,8 @@ def index_constructor_empty(
     return False
 
 def matmul(
-        lhs: tp.Union['Series', 'Frame', np.ndarray],
-        rhs: tp.Union['Series', 'Frame', np.ndarray],
+        lhs: tp.Union['Series', 'Frame', np.ndarray, tp.Sequence[float]],
+        rhs: tp.Union['Series', 'Frame', np.ndarray, tp.Sequence[float]],
         ) -> tp.Any: #tp.Union['Series', 'Frame']:
     '''
     Implementation of matrix multiplication for Series and Frame
@@ -406,7 +403,7 @@ def matmul(
 
 
 def axis_window_items( *,
-        source: tp.Union['Series', 'Frame'],
+        source: tp.Union['Series', 'Frame', 'Quilt'],
         size: int,
         axis: int = 0,
         step: int = 1,
@@ -418,20 +415,15 @@ def axis_window_items( *,
         size_increment: int = 0,
         as_array: bool = False,
         ) -> tp.Iterator[tp.Tuple[tp.Hashable, tp.Any]]:
-    '''Generator of index, window pairs pairs.
+    '''Generator of index, window pairs. When ndim is 2, axis 0 returns windows of rows, axis 1 returns windows of columns.
 
     Args:
-        size: integer greater than 0
-        step: integer greater than 0 to determine the step size between windows. A step of 1 shifts the window 1 data point; a step equal to window size results in non-overlapping windows.
-        window_sized: if True, windows that do not meet the size are skipped.
-        window_func: Array processor of window values, pre-function application; useful for applying weighting to the window.
-        window_valid: Function that, given an array window, returns True if the window meets requirements and should be returned.
-        label_shift: shift, relative to the right-most data point contained in the window, to derive the label to be paired with the window; e.g., to return the first label of the window, the shift will be the size minus one.
-        start_shift: shift from 0 to determine where the collection of windows begins.
-        size_increment: value to be added to each window aftert the first, so as to, in combination with setting the step size to 0, permit expanding windows.
         as_array: if True, the window is returned as an array instead of a SF object.
     '''
+    # see doc_str window for docs
+
     from static_frame.core.frame import Frame
+    from static_frame.core.quilt import Quilt
     from static_frame.core.series import Series
 
     if size <= 0:
@@ -440,6 +432,7 @@ def axis_window_items( *,
         raise RuntimeError('window step cannot be less than than 0')
 
     source_ndim = source.ndim
+    values: tp.Optional[np.ndarray] = None
 
     if source_ndim == 1:
         assert isinstance(source, Series) # for mypy
@@ -447,9 +440,10 @@ def axis_window_items( *,
         if as_array:
             values = source.values
     else:
-        assert isinstance(source, Frame) # for mypy
-        labels = source._index if axis == 0 else source._columns
-        if as_array:
+        labels = source._index if axis == 0 else source._columns #type: ignore
+
+        if isinstance(source, Frame) and axis == 0 and as_array:
+            # for a Frame, when collecting rows, it is more efficient to pre-consolidate blocks prior to slicing. Note that this results in the same block coercion necessary for each window (which is not the same for axis 1, where block coercion is not required)
             values = source._blocks.values
 
     if start_shift >= 0:
@@ -466,25 +460,27 @@ def axis_window_items( *,
         idx_right = idx_left + size - 1
 
         # floor idx_left at 0 so as to not wrap
-        idx_left_floored = max(idx_left, 0)
-        idx_right_floored = max(idx_right, -1) # will add one
+        idx_left_floored = idx_left if idx_left > 0 else 0
+        idx_right_floored = idx_right if idx_right > -1 else -1 # will add one
 
         key = slice(idx_left_floored, idx_right_floored + 1)
 
         if source_ndim == 1:
             if as_array:
-                window = values[key]
+                window = values[key] #type: ignore
             else:
                 window = source._extract_iloc(key)
         else:
-            if axis == 0:
-                if as_array:
+            if axis == 0: # extract rows
+                if as_array and values is not None:
                     window = values[key]
+                elif as_array:
+                    window = source._extract_array(key) #type: ignore
                 else: # use low level iloc selector
                     window = source._extract(row_key=key) #type: ignore
-            else:
+            else: # extract columns
                 if as_array:
-                    window = values[NULL_SLICE, key]
+                    window = source._extract_array(NULL_SLICE, key) #type: ignore
                 else:
                     window = source._extract(column_key=key) #type: ignore
 
@@ -493,7 +489,7 @@ def axis_window_items( *,
             idx_label = idx_right + label_shift
             if idx_label < 0: # do not wrap around
                 raise IndexError()
-            #if we cannot get a lable, the window is invalid
+            #if we cannot get a label, the window is invalid
             label = labels.iloc[idx_label]
         except IndexError: # an invalid label has to be dropped
             valid = False
@@ -512,8 +508,6 @@ def axis_window_items( *,
         size += size_increment
         count += 1
 
-        # import ipdb; ipdb.set_trace()
-
         if count > count_window_max or idx_left > idx_left_max or size < 0:
             break
 
@@ -567,7 +561,7 @@ def key_to_ascending_key(key: GetItemKeyType, size: int) -> GetItemKeyType:
         # array first as not truthy
         return np.sort(key, kind=DEFAULT_SORT_KIND)
 
-    if not key:
+    if not len(key): #type: ignore
         return key
 
     if isinstance(key, list):
@@ -693,6 +687,8 @@ def array_from_value_iter(
                 )
     return values
 
+#-------------------------------------------------------------------------------
+# utilities for binary operator applications with type blocks
 
 def apply_binary_operator(*,
         values: np.ndarray,
@@ -703,7 +699,6 @@ def apply_binary_operator(*,
     '''
     Utility to handle binary operator application.
     '''
-
     if (values.dtype.kind in DTYPE_STR_KINDS or
             (other_is_array and other.dtype.kind in DTYPE_STR_KINDS)):
         operator_name = operator.__name__
@@ -720,14 +715,21 @@ def apply_binary_operator(*,
         result = operator(values, other)
 
     if result is False or result is True:
-        # in comparison to Booleans, if values is of length 1 and a character type, we will get a Boolean back, not an array; this issues the following warning: FutureWarning: elementwise comparison failed; returning scalar instead, but in the future will perform elementwise comparison
-        # if the arguement is a tuple of length equal to an erray, NP will perform element wise comparison; but if the argment is a tuple of length greater or equal, each value in value will be compared to that tuple
-        result = np.full(values.shape, result)
+        if not other_is_array and (
+                isinstance(other, str) or not hasattr(other, '__len__')
+                ):
+            # only expand to the size of the array operand if we are comparing to an element
+            result = np.full(values.shape, result, dtype=DTYPE_BOOL)
+        elif other_is_array and other.size == 1:
+            # elements in arrays of 0 or more dimensions are acceptable; this is what NP does for arithmetic operators when the types are compatible
+            result = np.full(values.shape, result, dtype=DTYPE_BOOL)
+        else:
+            raise ValueError('operands could not be broadcast together')
+            # raise on unaligned shapes as is done for arithmetic operators
 
     result.flags.writeable = False
     return result
 
-
 def apply_binary_operator_blocks(*,
         values: tp.Iterable[np.ndarray],
         other: tp.Iterable[np.ndarray],
@@ -737,7 +739,6 @@ def apply_binary_operator_blocks(*,
     '''
     Application from iterators of arrays, to iterators of arrays.
     '''
-
     if apply_column_2d_filter:
         values = (column_2d_filter(op) for op in values)
         other = (column_2d_filter(op) for op in other)
@@ -750,6 +751,36 @@ def apply_binary_operator_blocks(*,
                 operator=operator,
                 )
 
+def apply_binary_operator_blocks_columnar(*,
+        values: tp.Iterable[np.ndarray],
+        other: np.ndarray,
+        operator: UFunc,
+    ) -> tp.Iterator[np.ndarray]:
+    '''
+    Application from iterators of arrays, to iterators of arrays. Will return iterator of all 1D arrays, as we will break down larger blocks in values into 1D arrays.
+
+    Args:
+        other: 1D array to be applied to each column of the blocks.
+    '''
+    assert other.ndim == 1
+    for block in values:
+        if block.ndim == 1:
+            yield apply_binary_operator(
+                    values=block,
+                    other=other,
+                    other_is_array=True,
+                    operator=operator,
+                    )
+        else:
+            for i in range(block.shape[1]):
+                yield apply_binary_operator(
+                        values=block[NULL_SLICE, i],
+                        other=other,
+                        other_is_array=True,
+                        operator=operator,
+                        )
+
+#-------------------------------------------------------------------------------
 
 def arrays_from_index_frame(
         container: 'Frame',
@@ -773,16 +804,21 @@ def key_from_container_key(
         key: GetItemKeyType,
         expand_iloc: bool = False,
         ) -> GetItemKeyType:
+    '''
+    Unpack selection values from another Index, Series, or ILoc selection.
+    '''
 
     from static_frame.core.index import Index
     from static_frame.core.index import ILoc
     from static_frame.core.series import Series
+    from static_frame.core.series import SeriesHE
 
     if isinstance(key, Index):
         # if an Index, we simply use the values of the index
         key = key.values
-    elif isinstance(key, Series):
-        if key.dtype == bool:
+    elif isinstance(key, Series) and key.__class__ is not SeriesHE:
+        # Series that are not hashable are unpacked into an array; SeriesHE can be used as a key
+        if key.dtype == DTYPE_BOOL:
             # if a Boolean series, sort and reindex
             if not key.index.equals(index):
                 key = key.reindex(index,
@@ -801,11 +837,9 @@ def key_from_container_key(
         key = array
 
     # detect and fail on Frame?
-
     return key
 
 
-
 #---------------------------------------------------------------------------
 
 def _index_many_to_one(
@@ -908,7 +942,7 @@ def index_many_set(
 def apex_to_name(
         rows: tp.Sequence[tp.Sequence[tp.Hashable]],
         depth_level: tp.Optional[DepthLevelSpecifier],
-        axis: int, # 0 is by row (for index, 1 is by column (for columns)
+        axis: int, # 0 is by row (for index), 1 is by column (for columns)
         axis_depth: int,
         ) -> NameType:
     '''
@@ -949,6 +983,59 @@ def apex_to_name(
     raise AxisInvalid(f'invalid axis: {axis}')
 
 
+def container_to_exporter_attr(container_type: tp.Type['Frame']) -> str:
+    from static_frame.core.frame import Frame
+    from static_frame.core.frame import FrameGO
+    from static_frame.core.frame import FrameHE
+
+    if container_type is Frame:
+        return 'to_frame'
+    elif container_type is FrameGO:
+        return 'to_frame_go'
+    elif container_type is FrameHE:
+        return 'to_frame_he'
+    raise NotImplementedError(f'no handling for {container_type}')
+
+
+def sort_index_for_order(
+        index: IndexBase,
+        ascending: bool,
+        kind: str,
+        key: tp.Optional[tp.Callable[[IndexBase], tp.Union[np.ndarray, IndexBase]]],
+        ) -> np.ndarray:
+    '''Return an integer array defing the new ordering.
+    '''
+    # cfs is container_for_sort
+    if key:
+        cfs = key(index)
+        cfs_is_array = isinstance(cfs, np.ndarray)
+        if cfs_is_array:
+            cfs_depth = 1 if cfs.ndim == 1 else cfs.shape[1]
+        else:
+            cfs_depth = cfs.depth
+        if len(cfs) != len(index):
+            raise RuntimeError('key function returned a container of invalid length')
+    else:
+        cfs = index
+        cfs_is_array = False
+        cfs_depth = cfs.depth
+
+    # argsort lets us do the sort once and reuse the results
+    if cfs_depth > 1:
+        if cfs_is_array:
+            values_for_lex = [cfs[:, i] for i in range(cfs.shape[1]-1, -1, -1)]
+        else: # cfs is an IndexHierarchy
+            values_for_lex = [cfs.values_at_depth(i)
+                    for i in range(cfs.depth-1, -1, -1)]
+        order = np.lexsort(values_for_lex)
+    else:
+        # depth is 1
+        v = cfs if cfs_is_array else cfs.values
+        order = np.argsort(v, kind=kind)
+
+    if not ascending:
+        order = order[::-1]
+    return order
 
 #-------------------------------------------------------------------------------
 
