@@ -3,6 +3,7 @@ from collections.abc import KeysView
 import operator as operator_mod
 from itertools import zip_longest
 from functools import reduce
+from copy import deepcopy
 
 import numpy as np
 
@@ -14,6 +15,7 @@ from static_frame.core.container import ContainerOperand
 from static_frame.core.container_util import apply_binary_operator
 from static_frame.core.container_util import matmul
 from static_frame.core.container_util import key_from_container_key
+from static_frame.core.container_util import sort_index_for_order
 
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
@@ -35,6 +37,7 @@ from static_frame.core.node_selector import TContainer
 from static_frame.core.node_str import InterfaceString
 
 from static_frame.core.util import array_shift
+from static_frame.core.util import array_sample
 from static_frame.core.util import array2d_to_tuples
 from static_frame.core.util import BOOL_TYPES
 from static_frame.core.util import CallableOrMapping
@@ -75,6 +78,8 @@ from static_frame.core.util import to_datetime64
 from static_frame.core.util import UFunc
 from static_frame.core.util import ufunc_axis_skipna
 from static_frame.core.util import union1d
+from static_frame.core.util import PositionsAllocator
+from static_frame.core.util import array_deepcopy
 
 if tp.TYPE_CHECKING:
     import pandas #pylint: disable=W0611 #pragma: no cover
@@ -89,7 +94,7 @@ class ILocMeta(type):
 
     def __getitem__(cls,
             key: GetItemKeyType
-            ) -> tp.Iterable[GetItemKeyType]:
+            ) -> 'ILoc':
         return cls(key) #type: ignore
 
 class ILoc(metaclass=ILocMeta):
@@ -125,10 +130,11 @@ class LocMap:
                 yield None
 
             elif isinstance(attr, np.datetime64):
+                assert labels is not None
                 # if a datetime, we assume that the labels are ordered;
-                if attr.dtype == labels.dtype: #type: ignore
+                if attr.dtype == labels.dtype:
                     if field != SLICE_STEP_ATTR:
-                        pos: int = label_to_pos(attr)
+                        pos: tp.Optional[int] = label_to_pos(attr)
                         if pos is None:
                             # if same type, and that atter is not in labels, we fail, just as we do in then non-datetime64 case. Only when datetimes are given in a different unit are we "loose" about matching.
                             raise LocInvalid('Invalid loc given in a slice', attr, field)
@@ -136,11 +142,11 @@ class LocMap:
                         pos = attr # should be an integer
 
                     if field == SLICE_STOP_ATTR:
-                        pos += 1 # stop is inclusive
+                        pos += 1 #type: ignore  # stop is inclusive
 
                 elif field == SLICE_START_ATTR:
                     # convert to the type of the atrs; this should get the relevant start
-                    pos = label_to_pos(attr.astype(labels.dtype)) #type: ignore
+                    pos: tp.Optional[int] = label_to_pos(attr.astype(labels.dtype)) #type: ignore
                     if pos is None: # we did not find a start position
                         matches = np.flatnonzero(labels.astype(attr.dtype) == attr)
                         if len(matches):
@@ -152,7 +158,7 @@ class LocMap:
                     # convert labels to the slice attr value, compare, then get last
                     # add one, as this is an inclusive stop
                     # pos = np.flatnonzero(labels.astype(attr.dtype) == attr)[-1] + 1
-                    matches = np.flatnonzero(labels.astype(attr.dtype) == attr) #type: ignore
+                    matches = np.flatnonzero(labels.astype(attr.dtype) == attr)
                     if len(matches):
                         pos = matches[-1] + 1
                     else:
@@ -179,7 +185,7 @@ class LocMap:
 
                 if field == SLICE_STOP_ATTR:
                     # loc selections are inclusive, so iloc gets one more
-                    pos += 1
+                    pos += 1 #type: ignore
 
                 yield pos
 
@@ -196,8 +202,8 @@ class LocMap:
         Note: all SF objects (Series, Index) need to be converted to basic types before being passed as `key` to this function.
 
         Args:
-            offset: in the contect of an IndexHierarchical, the iloc positions returned from this funcition need to be shifted.
-            partial_selection: if True and key is an iterable of labels that includes lables not in the mapping, available matches will be returned rather than raising.
+            offset: in the context of an IndexHierarchical, the iloc positions returned from this funcition need to be shifted.
+            partial_selection: if True and key is an iterable of labels that includes labels not in the mapping, available matches will be returned rather than raising.
         Returns:
             An integer mapped slice, or GetItemKey type that is based on integers, compatible with TypeBlocks
         '''
@@ -207,7 +213,7 @@ class LocMap:
 
         if isinstance(key, slice):
             if offset_apply and key == NULL_SLICE:
-                # when offset is defined (even if it is zero), null slice is not sufficiently specific; need to convert to an explict slice relative to the offset
+                # when offset is defined (even if it is zero), null slice is not sufficiently specific; need to convert to an explicit slice relative to the offset
                 return slice(offset, len(positions) + offset) #type: ignore
             try:
                 return slice(*cls.map_slice_args(
@@ -277,23 +283,6 @@ def mutable_immutable_index_filter(
         return index._MUTABLE_CONSTRUCTOR(index)
     return index.__class__(index) # create new instance
 
-#-------------------------------------------------------------------------------
-
-class PositionsAllocator:
-
-    _size: int = 0
-    _array: np.ndarray = np.arange(_size, dtype=DTYPE_INT_DEFAULT)
-    _array.flags.writeable = False
-
-    @classmethod
-    def get(cls, size: int) -> np.ndarray:
-        if size > cls._size:
-            cls._size = size * 2
-            cls._array = np.arange(cls._size, dtype=DTYPE_INT_DEFAULT)
-            cls._array.flags.writeable = False
-        # slices of immutable arrays are immutable
-        return cls._array[:size]
-
 #-------------------------------------------------------------------------------
 _INDEX_SLOTS = (
         '_map',
@@ -409,7 +398,7 @@ class Index(IndexBase):
 
         # resolve the targetted labels dtype, by lookin at the class attr _DTYPE and/or the passed dtype argument
         if dtype is None:
-            dtype_extract = self._DTYPE # set in some specialized Index classes
+            dtype_extract = self._DTYPE # set in some specialized Index sub-classes
         else: # passed dtype is not None
             if is_typed and dtype != self._DTYPE:
                 # NOTE: should never get to this branch, as derived Index classes that set _DTYPE remove dtype from __init__
@@ -485,7 +474,6 @@ class Index(IndexBase):
             raise ErrorInitIndex('invalid label dtype for this Index', #pragma: no cover
                     self._labels.dtype, self._DTYPE)
 
-
     #---------------------------------------------------------------------------
     def __setstate__(self, state: tp.Tuple[None, tp.Dict[str, tp.Any]]) -> None:
         '''
@@ -495,6 +483,35 @@ class Index(IndexBase):
             setattr(self, key, value)
         self._labels.flags.writeable = False
 
+    def __deepcopy__(self: I, memo: tp.Dict[int, tp.Any]) -> I:
+        if self._recache:
+            self._update_array_cache()
+
+        obj = self.__new__(self.__class__)
+        obj._map = deepcopy(self._map, memo) #type: ignore
+        obj._labels = array_deepcopy(self._labels, memo) #type: ignore
+        obj._positions = PositionsAllocator.get(len(self._labels)) #type: ignore
+        obj._recache = False
+        obj._name = self._name # should be hashable/immutable
+
+        memo[id(self)] = obj
+        return obj #type: ignore
+
+    def __copy__(self: I) -> I:
+        '''
+        Return shallow copy of this Index.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        return self.__class__(self, name=self._name)
+
+    def copy(self: I) -> I:
+        '''
+        Return shallow copy of this Index.
+        '''
+        return self.__copy__() #type: ignore
+
     #---------------------------------------------------------------------------
     # name interface
 
@@ -521,12 +538,12 @@ class Index(IndexBase):
     # # on Index, getitem is an iloc selector; on Series, getitem is a loc selector; for this extraction interface, we do not implement a getitem level function (using iloc would be consistent), as it is better to be explicit between iloc loc
 
     def _iter_label(self,
-            depth_level: DepthLevelSpecifier = 0
+            depth_level: tp.Optional[DepthLevelSpecifier] = None
             ) -> tp.Iterator[tp.Hashable]:
         yield from self._labels
 
     def _iter_label_items(self,
-            depth_level: DepthLevelSpecifier = 0
+            depth_level: tp.Optional[DepthLevelSpecifier] = None
             ) -> tp.Iterator[tp.Tuple[int, tp.Hashable]]:
         yield from zip(self._positions, self._labels)
 
@@ -613,20 +630,20 @@ class Index(IndexBase):
             self._update_array_cache()
         return tp.cast(int, self._labels.nbytes)
 
-    def __bool__(self) -> bool:
-        '''
-        True if this container has size.
-        '''
-        if self._recache:
-            self._update_array_cache()
-        return bool(self._labels.size)
+    # def __bool__(self) -> bool:
+    #     '''
+    #     True if this container has size.
+    #     '''
+    #     if self._recache:
+    #         self._update_array_cache()
+    #     return bool(self._labels.size)
 
     #---------------------------------------------------------------------------
     # set operations
 
     def _ufunc_set(self: I,
             func: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray],
-            other: tp.Union['IndexBase', 'Series']
+            other: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]
             ) -> I:
         '''
         Utility function for preparing and collecting values for Indices to produce a new Index.
@@ -653,11 +670,8 @@ class Index(IndexBase):
         elif isinstance(other, IndexBase):
             operand = other.values
             assume_unique = True # can always assume unique
-        elif isinstance(other, ContainerOperand): # TODO 0.7: use iterable to array force user to provide values
-            operand = other.values
-            assume_unique = False
         else:
-            raise NotImplementedError(f'no support for {other}')
+            operand, assume_unique = iterable_to_array_1d(other)
 
         cls = self.__class__
 
@@ -763,7 +777,6 @@ class Index(IndexBase):
                 blocks_to_container=blocks_to_container,
                 )
 
-
     #---------------------------------------------------------------------------
 
     def _update_array_cache(self) -> None:
@@ -864,16 +877,6 @@ class Index(IndexBase):
 
     #---------------------------------------------------------------------------
 
-    def copy(self: I) -> I:
-        '''
-        Return a new Index.
-        '''
-        # this is not a complete deepcopy, as _labels here is an immutable np array (a new map will be created); if this is an IndexGO, we will pass the cached, immutable NP array
-        if self._recache:
-            self._update_array_cache()
-
-        return self.__class__(self, name=self._name)
-
     def relabel(self, mapper: 'RelabelInput') -> 'Index':
         '''
         Return a new Index with labels replaced by the callable or mapping; order will be retained. If a mapping is used, the mapping need not map all origin keys.
@@ -922,7 +925,7 @@ class Index(IndexBase):
 
         key = key_from_container_key(self, key)
 
-        if self._map is None: # loc_is_iloc
+        if self._map is None and offset is None: # loc_is_iloc
             if isinstance(key, np.ndarray):
                 if key.dtype == bool:
                     return key
@@ -934,11 +937,39 @@ class Index(IndexBase):
                 key = slice_to_inclusive_slice(key)
             return key
 
+        if self._map is None and offset is not None: # loc_is_iloc
+            if isinstance(key, slice):
+                if key == NULL_SLICE:
+                    return slice(offset, self.__len__() + offset)
+
+                key = slice_to_inclusive_slice(key)
+
+                def slice_attrs() -> tp.Iterator[int]:
+                    for attr in SLICE_ATTRS:
+                        if attr != SLICE_STEP_ATTR:
+                            yield getattr(key, attr) + offset
+                        else:
+                            yield getattr(key, attr)
+
+                return slice(*slice_attrs())
+
+            if isinstance(key, np.ndarray):
+                if key.dtype == DTYPE_BOOL:
+                    return self._positions[key] + offset
+                if key.dtype != DTYPE_INT_DEFAULT:
+                    key = key.astype(DTYPE_INT_DEFAULT)
+                return key + offset
+
+            if isinstance(key, list):
+               return [k + offset for k in key]
+            # a single element
+            return key + offset # type: ignore
+
         if key_transform:
             key = key_transform(key)
 
         return LocMap.loc_to_iloc(
-                label_to_pos=self._map,
+                label_to_pos=self._map, #type: ignore
                 labels=self._labels,
                 positions=self._positions, # always an np.ndarray
                 key=key,
@@ -1003,11 +1034,17 @@ class Index(IndexBase):
             other: tp.Any
             ) -> np.ndarray:
         '''
-        Binary operators applied to an index always return an NP array. This deviates from Pandas, where some operations (multipling an int index by an int) result in a new Index, while other operations result in a np.array (using == on two Index).
+        Binary operators applied to an index always return an NP array. This deviates from Pandas, where some operations (multiplying an int index by an int) result in a new Index, while other operations result in a np.array (using == on two Index).
         '''
+        from static_frame.core.series import Series
+        from static_frame.core.frame import Frame
+
         if self._recache:
             self._update_array_cache()
 
+        if isinstance(other, (Series, Frame)):
+            raise ValueError('cannot use labelled container as an operand.')
+
         values = self._labels
         other_is_array = False
 
@@ -1084,7 +1121,7 @@ class Index(IndexBase):
         if self._map is None: # loc_is_iloc
             if isinstance(value, INT_TYPES):
                 return value >= 0 and value < len(self) #type: ignore
-            return False
+            return False #type: ignore [unreachable]
         return self._map.__contains__(value) #type: ignore
 
 
@@ -1134,7 +1171,7 @@ class Index(IndexBase):
 
         # NOTE: will only be False, or an array
         if eq is False:
-            return eq #type: ignore
+            return eq
 
         if skipna:
             isna_both = (isna_array(self.values, include_none=False)
@@ -1145,21 +1182,22 @@ class Index(IndexBase):
             return False
         return True
 
+    @doc_inject(selector='sort')
     def sort(self,
             ascending: bool = True,
-            kind: str = DEFAULT_SORT_KIND) -> 'Index':
+            kind: str = DEFAULT_SORT_KIND,
+            key: tp.Optional[tp.Callable[['Index'], tp.Union[np.ndarray, 'Index']]] = None,
+            ) -> 'Index':
         '''Return a new Index with the labels sorted.
 
         Args:
-            kind: Sort algorithm passed to NumPy.
+            ascending: {ascending}
+            kind: {kind}
+            key: {key}
         '''
-        # force usage of property for caching
-        v = np.sort(self.values, kind=kind)
-        if not ascending:
-            v = v[::-1]
+        order = sort_index_for_order(self, kind=kind, ascending=ascending, key=key) #type: ignore [arg-type]
 
-        v.flags.writeable = False
-        return self.__class__(v, name=self._name)
+        return self._extract_iloc(order) #type: ignore [return-value]
 
     def isin(self, other: tp.Iterable[tp.Any]) -> np.ndarray:
         '''
@@ -1205,6 +1243,20 @@ class Index(IndexBase):
 
         return self.__class__(assigned, name=self._name)
 
+    def _sample_and_key(self,
+            count: int = 1,
+            *,
+            seed: tp.Optional[int] = None,
+            ) -> tp.Tuple['Index', np.ndarray]:
+        # NOTE: base class defines pubic method
+        # force usage of property for cache update
+        # sort positions to avoid uncomparable objects
+        key = array_sample(self.positions, count=count, seed=seed, sort=True)
+
+        values = self.values[key]
+        values.flags.writeable = False
+        return self.__class__(values, name=self._name), key
+
 
     #---------------------------------------------------------------------------
     # export
@@ -1217,14 +1269,14 @@ class Index(IndexBase):
         return Series(self.values, name=self._name)
 
 
-    def add_level(self, level: tp.Hashable) -> 'IndexHierarchy':
+    def level_add(self, level: tp.Hashable) -> 'IndexHierarchy':
         '''Return an IndexHierarchy with an added root level.
         '''
         from static_frame import IndexHierarchy
         from static_frame import IndexHierarchyGO
 
         cls = IndexHierarchy if self.STATIC else IndexHierarchyGO
-        return cls.from_tree({level: self.values})
+        return cls.from_tree({level: self.values}, name=self._name)
 
 
     def to_pandas(self) -> 'pandas.Index':
@@ -1254,13 +1306,31 @@ class _IndexGOMixin:
     __slots__ = () # define in derived class
 
     _map: tp.Optional[AutoMap]
+    _labels: np.ndarray
+    _positions: np.ndarray
     _labels_mutable: tp.List[tp.Hashable]
     _labels_mutable_dtype: np.dtype
     _positions_mutable_count: int
-    _positions: np.ndarray
-    _labels: np.ndarray
 
+    #---------------------------------------------------------------------------
+    def __deepcopy__(self: I, memo: tp.Dict[int, tp.Any]) -> I: #type: ignore
+        if self._recache:
+            self._update_array_cache()
 
+        obj = self.__new__(self.__class__)
+        obj._map = deepcopy(self._map, memo) #type: ignore
+        obj._labels = array_deepcopy(self._labels, memo) #type: ignore
+        obj._positions = PositionsAllocator.get(len(self._labels)) #type: ignore
+        obj._recache = False
+        obj._name = self._name # should be hashable/immutable
+        obj._labels_mutable = deepcopy(self._labels_mutable, memo) #type: ignore
+        obj._labels_mutable_dtype = deepcopy(self._labels_mutable_dtype, memo) #type: ignore
+        obj._positions_mutable_count = self._positions_mutable_count #type: ignore
+
+        memo[id(self)] = obj
+        return obj #type: ignore
+
+    #---------------------------------------------------------------------------
     def _extract_labels(self,
             mapping: tp.Optional[tp.Dict[tp.Hashable, int]],
             labels: np.ndarray,
