@@ -4,7 +4,7 @@ import typing as tp
 
 from static_frame.core.index import Index
 from static_frame.core.index import IndexGO
-from static_frame.core.index import PositionsAllocator
+from static_frame.core.util import PositionsAllocator
 from static_frame.core.index_base import IndexBase  # pylint: disable = W0611
 from static_frame.core.util import DTYPE_INT_DEFAULT
 from static_frame.core.util import IndexConstructor
