@@ -1,6 +1,6 @@
 import typing as tp
-import numpy as np
 
+import numpy as np
 
 from static_frame.core.container import ContainerOperand
 from static_frame.core.display import Display
@@ -8,21 +8,21 @@ from static_frame.core.display import DisplayActive
 from static_frame.core.display_config import DisplayConfig
 from static_frame.core.display_config import DisplayFormats
 from static_frame.core.doc_str import doc_inject
+from static_frame.core.exception import ErrorInitIndex
+from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import NameType
 from static_frame.core.util import PathSpecifierOrFileLike
 from static_frame.core.util import UFunc
 from static_frame.core.util import write_optional_file
-from static_frame.core.util import DepthLevelSpecifier
-from static_frame.core.exception import ErrorInitIndex
 
 
 
 if tp.TYPE_CHECKING:
     import pandas #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.series import Series #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.index_auto import RelabelInput #pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.series import Series #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.index_auto import RelabelInput #pylint: disable=W0611,C0412 #pragma: no cover
 
 I = tp.TypeVar('I', bound='IndexBase')
 
@@ -52,8 +52,8 @@ class IndexBase(ContainerOperand):
     __add__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __sub__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __mul__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __matmul__: tp.Callable[['IndexBase', tp.Any], np.ndarray] #type: ignore
-    __truediv__: tp.Callable[['IndexBase', tp.Any], np.ndarray] #type: ignore
+    __matmul__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
+    __truediv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __floordiv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __mod__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     # __divmod__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
@@ -72,7 +72,7 @@ class IndexBase(ContainerOperand):
     __radd__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __rsub__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __rmul__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
-    __rtruediv__: tp.Callable[['IndexBase', tp.Any], np.ndarray] #type: ignore
+    __rtruediv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     __rfloordiv__: tp.Callable[['IndexBase', tp.Any], np.ndarray]
     # __len__: tp.Callable[['IndexBase'], int]
 
@@ -100,7 +100,7 @@ class IndexBase(ContainerOperand):
         '''
         import pandas
         if not isinstance(value, pandas.Index):
-            raise ErrorInitIndex('from_pandas must be called with a Pandas object')
+            raise ErrorInitIndex(f'from_pandas must be called with a Pandas Index object, not: {type(value)}')
 
         from static_frame import Index
         from static_frame import IndexGO
@@ -193,18 +193,44 @@ class IndexBase(ContainerOperand):
     def _drop_iloc(self: I, key: GetItemKeyType) -> I:
         raise NotImplementedError() #pragma: no cover
 
-    def roll(self: I, shift: int) -> I:
-        raise NotImplementedError() #pragma: no cover
-
     def isin(self, other: tp.Iterable[tp.Any]) -> np.ndarray:
         raise NotImplementedError() #pragma: no cover
 
-    def add_level(self, level: tp.Hashable) -> 'IndexHierarchy':
+    def roll(self: I, shift: int) -> I:
+        raise NotImplementedError() #pragma: no cover
+
+    def fillna(self: I, value: tp.Any) -> I:
+        raise NotImplementedError() #pragma: no cover
+
+    def _sample_and_key(self: I,
+            count: int = 1,
+            *,
+            seed: tp.Optional[int] = None,
+            ) -> tp.Tuple[I, np.ndarray]:
+        raise NotImplementedError() #pragma: no cover
+
+    def level_add(self, level: tp.Hashable) -> 'IndexHierarchy':
         raise NotImplementedError() #pragma: no cover
 
     def display(self, config: tp.Optional[DisplayConfig] = None) -> Display:
         raise NotImplementedError()
 
+    #---------------------------------------------------------------------------
+    @doc_inject(selector='sample')
+    def sample(self: I,
+            count: int = 1,
+            *,
+            seed: tp.Optional[int] = None,
+            ) -> I:
+        '''{doc}
+
+        Args:
+            {count}
+            {seed}
+        '''
+        container, _ = self._sample_and_key(count=count, seed=seed)
+        return container
+
     #---------------------------------------------------------------------------
 
     def loc_to_iloc(self,
@@ -278,11 +304,11 @@ class IndexBase(ContainerOperand):
 
     def _ufunc_set(self: I,
             func: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray],
-            other: tp.Union['IndexBase', 'Series']
+            other: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]
             ) -> I:
         raise NotImplementedError() #pragma: no cover
 
-    def intersection(self: I, *others: tp.Union['IndexBase', 'Series']) -> I:
+    def intersection(self: I, *others: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
         '''
         Perform intersection with one or many Index, container, or NumPy array. Identical comparisons retain order.
         '''
@@ -296,7 +322,7 @@ class IndexBase(ContainerOperand):
             post = post._ufunc_set(func, other)
         return post
 
-    def union(self: I, *others: 'IndexBase') -> I:
+    def union(self: I, *others: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
         '''
         Perform union with another Index, container, or NumPy array. Identical comparisons retain order.
         '''
@@ -310,7 +336,7 @@ class IndexBase(ContainerOperand):
         return post
 
 
-    def difference(self: I, other: 'IndexBase') -> I:
+    def difference(self: I, other: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]) -> I:
         '''
         Perform difference with another Index, container, or NumPy array. Retains order.
         '''
@@ -346,8 +372,6 @@ class IndexBase(ContainerOperand):
                 size_one_unity=size_one_unity
                 )
 
-
-
     #---------------------------------------------------------------------------
     # exporters
 
