@@ -1,17 +1,17 @@
 import typing as tp
 from itertools import chain
 from ast import literal_eval
+from copy import deepcopy
 
 import numpy as np
 
 
 from static_frame.core.array_go import ArrayGO
-from static_frame.core.container import ContainerOperand
-from static_frame.core.container_util import apply_binary_operator
 from static_frame.core.container_util import index_from_optional_constructor
 from static_frame.core.container_util import matmul
 from static_frame.core.container_util import rehierarch_from_type_blocks
 from static_frame.core.container_util import key_from_container_key
+from static_frame.core.container_util import sort_index_for_order
 
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
@@ -25,7 +25,7 @@ from static_frame.core.index import ILoc
 
 from static_frame.core.index import Index
 from static_frame.core.index import IndexGO
-from static_frame.core.index import PositionsAllocator
+from static_frame.core.util import PositionsAllocator
 from static_frame.core.index import mutable_immutable_index_filter
 from static_frame.core.index_base import IndexBase
 from static_frame.core.index_level import IndexLevel
@@ -40,6 +40,7 @@ from static_frame.core.node_selector import InterfaceAsType
 from static_frame.core.node_selector import InterfaceGetItem
 from static_frame.core.node_selector import TContainer
 from static_frame.core.node_str import InterfaceString
+from static_frame.core.node_transpose import InterfaceTranspose
 
 from static_frame.core.type_blocks import TypeBlocks
 
@@ -47,6 +48,7 @@ from static_frame.core.util import DEFAULT_SORT_KIND
 from static_frame.core.util import DepthLevelSpecifier
 from static_frame.core.util import DtypeSpecifier
 from static_frame.core.util import EMPTY_TUPLE
+from static_frame.core.util import DTYPE_BOOL
 
 from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import IndexConstructor
@@ -62,12 +64,15 @@ from static_frame.core.util import NULL_SLICE
 from static_frame.core.util import setdiff2d
 from static_frame.core.util import UFunc
 from static_frame.core.util import union2d
+from static_frame.core.util import array2d_to_tuples
+from static_frame.core.util import iterable_to_array_2d
+from static_frame.core.util import array_sample
 
 if tp.TYPE_CHECKING:
     from pandas import DataFrame #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.frame import Frame #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.frame import FrameGO #pylint: disable=W0611 #pragma: no cover
-    from static_frame.core.series import Series #pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.frame import Frame #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.frame import FrameGO #pylint: disable=W0611,C0412 #pragma: no cover
+    from static_frame.core.series import Series #pylint: disable=W0611,C0412 #pragma: no cover
 
 IH = tp.TypeVar('IH', bound='IndexHierarchy')
 
@@ -365,7 +370,10 @@ class IndexHierarchy(IndexBase):
             if len(parts) <= 1:
                 raise RuntimeError(f'Could not not parse more than one label from delimited string: {label}')
 
-            return tuple(literal_eval(p) for p in parts)
+            try:
+                return tuple(literal_eval(p) for p in parts)
+            except ValueError as e:
+                raise ValueError('A label is malformed. This is most likely due to not quoting a string label') from e
 
         return cls.from_labels(
                 (to_label(label) for label in labels),
@@ -519,6 +527,41 @@ class IndexHierarchy(IndexBase):
         self._recache = self._blocks is None
         self._name = None if name is NAME_DEFAULT else name_filter(name)
 
+    #---------------------------------------------------------------------------
+    def __deepcopy__(self: IH, memo: tp.Dict[int, tp.Any]) -> IH:
+        if self._recache:
+            self._update_array_cache()
+
+        obj = self.__new__(self.__class__)
+        obj._levels = deepcopy(self._levels, memo)
+        obj._blocks = deepcopy(self._blocks, memo)
+        obj._recache = False
+        obj._name = self._name # should be hashable/immutable
+
+        memo[id(self)] = obj
+        return obj #type: ignore
+
+    def __copy__(self: IH) -> IH:
+        '''
+        Return a shallow copy of this IndexHierarchy.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        blocks = self._blocks.copy()
+        return self.__class__(
+                levels=self._levels, # immutable, can re-use
+                name=self._name,
+                blocks=blocks,
+                own_blocks=True
+                )
+
+    def copy(self: IH) -> IH:
+        '''
+        Return a shallow copy of this IndexHierarchy.
+        '''
+        return self.__copy__()
+
     #---------------------------------------------------------------------------
     # name interface
 
@@ -541,16 +584,36 @@ class IndexHierarchy(IndexBase):
     def iloc(self) -> InterfaceGetItem['IndexHierarchy']:
         return InterfaceGetItem(self._extract_iloc) #type: ignore
 
-    # HOTE: this is only nodes, not a the full realization; probably need both
+
     def _iter_label(self,
-            depth_level: int = 0,
+            depth_level: tp.Optional[DepthLevelSpecifier] = None,
             ) -> tp.Iterator[tp.Hashable]:
-        yield from self._levels.label_nodes_at_depth(depth_level=depth_level)
+
+        if depth_level is None: # default to full labels
+            depth_level = list(range(self.depth))
+
+        # if no type blocks, use a levels
+        if self._recache:
+            if isinstance(depth_level, int):
+                yield from self._levels.labels_at_depth(depth_level=depth_level)
+            else:
+                yield from zip(
+                        *(self._levels.labels_at_depth(depth_level=d) for d in depth_level)
+                        )
+        else:
+            if isinstance(depth_level, int):
+                yield from self._blocks._extract_array(column_key=depth_level)
+            else:
+                yield from array2d_to_tuples(
+                        self._blocks._extract_array(column_key=depth_level)
+                        )
 
     def _iter_label_items(self,
-            depth_level: int = 0,
+            depth_level: tp.Optional[DepthLevelSpecifier] = None,
             ) -> tp.Iterator[tp.Tuple[int, tp.Hashable]]:
-        yield from enumerate(self._levels.label_nodes_at_depth(depth_level=depth_level))
+
+        yield from enumerate(self._iter_label(depth_level=depth_level))
+
 
     @property
     def iter_label(self) -> IterNodeDepthLevel[tp.Any]:
@@ -609,6 +672,16 @@ class IndexHierarchy(IndexBase):
                 )
 
 
+    @property
+    def via_T(self) -> InterfaceTranspose['IndexHierarchy']:
+        '''
+        Interface for using binary operators with one-dimensional sequences, where the opperand is applied column-wise.
+        '''
+        return InterfaceTranspose(
+                container=self,
+                )
+
+
     #---------------------------------------------------------------------------
 
     def _update_array_cache(self) -> None:
@@ -696,13 +769,13 @@ class IndexHierarchy(IndexBase):
             self._update_array_cache()
         return self._blocks.nbytes
 
-    def __bool__(self) -> bool:
-        '''
-        True if this container has size.
-        '''
-        if self._recache:
-            return bool(self._levels.__len__()) and bool(self._levels.depth)
-        return bool(self._blocks.size)
+    # def __bool__(self) -> bool:
+    #     '''
+    #     True if this container has size.
+    #     '''
+    #     if self._recache:
+    #         return bool(self._levels.__len__()) and bool(self._levels.depth)
+    #     return bool(self._blocks.size)
 
     #---------------------------------------------------------------------------
 
@@ -726,7 +799,7 @@ class IndexHierarchy(IndexBase):
         if self._recache:
             self._update_array_cache()
 
-        sub_display = None
+        sub_display: tp.Optional[Display] = None
 
         header_sub: tp.Optional[str]
         header: tp.Optional[DisplayHeader]
@@ -760,7 +833,7 @@ class IndexHierarchy(IndexBase):
 
     def _ufunc_set(self,
             func: tp.Callable[[np.ndarray, np.ndarray, bool], np.ndarray],
-            other: tp.Union['IndexBase', 'Series']
+            other: tp.Union['IndexBase', tp.Iterable[tp.Hashable]]
             ) -> 'IndexHierarchy':
         '''
         Utility function for preparing and collecting values for Indices to produce a new Index.
@@ -785,11 +858,9 @@ class IndexHierarchy(IndexBase):
         elif isinstance(other, IndexBase):
             operand = other.values
             assume_unique = True # can always assume unique
-        elif isinstance(other, ContainerOperand): # TODO 0.7: use iterable to array force user to provide values
-            operand = other.values
-            assume_unique = False
         else:
-            raise NotImplementedError(f'no support for {other}')
+            operand = iterable_to_array_2d(other) #type: ignore
+            assume_unique = False
 
         both_sized = len(operand) > 0 and len(self) > 0
 
@@ -923,22 +994,6 @@ class IndexHierarchy(IndexBase):
         return Series(self._levels.index_types(), index=labels)
 
     #---------------------------------------------------------------------------
-
-    def copy(self: IH) -> IH:
-        '''
-        Return a new IndexHierarchy. This is not a deep copy.
-        '''
-        if self._recache:
-            self._update_array_cache()
-
-        blocks = self._blocks.copy()
-        return self.__class__(
-                levels=self._levels,
-                name=self._name,
-                blocks=blocks,
-                own_blocks=True
-                )
-
     def relabel(self, mapper: RelabelInput) -> 'IndexHierarchy':
         '''
         Return a new IndexHierarchy with labels replaced by the callable or mapping; order will be retained. If a mapping is used, the mapping should map tuple representation of labels, and need not map all origin keys.
@@ -1003,14 +1058,16 @@ class IndexHierarchy(IndexBase):
         elif isinstance(key, IndexHierarchy):
             # default iteration of IH is as tuple
             return [self._levels.leaf_loc_to_iloc(k) for k in key]
-
-        key = key_from_container_key(self, key)
+        elif isinstance(key, np.ndarray) and key.dtype == DTYPE_BOOL:
+            return self.positions[key]
 
         if isinstance(key, HLoc):
             # unpack any Series, Index, or ILoc into the context of this IndexHierarchy
             key = HLoc(tuple(
                     key_from_container_key(self, k, expand_iloc=True)
                     for k in key))
+        else:
+            key = key_from_container_key(self, key)
 
         return self._levels.loc_to_iloc(key)
 
@@ -1078,39 +1135,39 @@ class IndexHierarchy(IndexBase):
     def _ufunc_binary_operator(self, *,
             operator: UFunc,
             other: tp.Any,
+            axis: int = 0,
             ) -> np.ndarray:
         '''
-        Binary operators applied to an index always return an NP array. This deviates from Pandas, where some operations (multipling an int index by an int) result in a new Index, while other operations result in a np.array (using == on two Index).
+        Binary operators applied to an index always return an NP array. This deviates from Pandas, where some operations (multiplying an int index by an int) result in a new Index, while other operations result in a np.array (using == on two Index).
         '''
+        from static_frame.core.series import Series
+        from static_frame.core.frame import Frame
+
         if self._recache:
             self._update_array_cache()
 
-        # NOTE: might use TypeBlocks._ufunc_binary_operator
-        values = self._blocks.values
-
-        other_is_array = False
-        if isinstance(other, Index):
-            # if this is a 1D index, must rotate labels before using an operator
-            other = other.values.reshape((len(other), 1)) # operate on labels to labels
-            other_is_array = True
-        elif isinstance(other, IndexHierarchy):
-            # already 2D
-            other = other.values # operate on labels to labels
-            other_is_array = True
-        elif isinstance(other, np.ndarray):
-            other_is_array = True
+        if isinstance(other, (Series, Frame)):
+            raise ValueError('cannot use labelled container as an operand.')
 
         if operator.__name__ == 'matmul':
-            return matmul(values, other)
+            return matmul(self._blocks.values, other)
         elif operator.__name__ == 'rmatmul':
-            return matmul(other, values)
+            return matmul(other, self._blocks.values)
 
-        return apply_binary_operator(
-                values=values,
-                other=other,
-                other_is_array=other_is_array,
+        if isinstance(other, Index):
+            other = other.values
+        elif isinstance(other, IndexHierarchy):
+            if other._recache:
+                other._update_array_cache()
+            other = other._blocks
+
+        tb = self._blocks._ufunc_binary_operator(
                 operator=operator,
+                other=other,
+                axis=axis,
                 )
+        return tb.values
+
 
     def _ufunc_axis_skipna(self, *,
             axis: int,
@@ -1128,7 +1185,7 @@ class IndexHierarchy(IndexBase):
         if self._recache:
             self._update_array_cache()
 
-        dtype = None if not dtypes else dtypes[0]
+        dtype = None if not dtypes else dtypes[0] # must be a tuple
         values = self._blocks.values
 
         if skipna:
@@ -1211,24 +1268,24 @@ class IndexHierarchy(IndexBase):
                 skipna=skipna,
                 )
 
-
+    @doc_inject(selector='sort')
     def sort(self: IH,
+            *,
             ascending: bool = True,
-            kind: str = DEFAULT_SORT_KIND
+            kind: str = DEFAULT_SORT_KIND,
+            key: tp.Optional[tp.Callable[['IndexHierarchy'], tp.Union[np.ndarray, 'IndexHierarchy']]] = None,
             ) -> IH:
         '''Return a new Index with the labels sorted.
 
         Args:
-            kind: Sort algorithm passed to NumPy.
+            ascending: {ascending}
+            kind: {kind}
+            key: {key}
         '''
         if self._recache:
             self._update_array_cache()
 
-        v = self._blocks.values
-        order = np.lexsort([v[:, i] for i in range(v.shape[1]-1, -1, -1)])
-
-        if not ascending:
-            order = order[::-1]
+        order = sort_index_for_order(self, kind=kind, ascending=ascending, key=key) #type: ignore [arg-type]
 
         blocks = self._blocks._extract(row_key=order)
         index_constructors = tuple(self._levels.index_types())
@@ -1298,6 +1355,27 @@ class IndexHierarchy(IndexBase):
                 )
 
 
+    def _sample_and_key(self,
+            count: int = 1,
+            *,
+            seed: tp.Optional[int] = None,
+            ) -> tp.Tuple['IndexHierarchy', np.ndarray]:
+        if self._recache:
+            self._update_array_cache()
+
+        # sort to ensure hierarchability
+        key = array_sample(self.positions, count=count, seed=seed, sort=True)
+        blocks = self._blocks._extract(row_key=key)
+
+        index_constructors = tuple(self._levels.index_types())
+
+        container = self.__class__._from_type_blocks(blocks,
+                index_constructors=index_constructors,
+                name=self._name,
+                own_blocks=True
+                )
+        return container, key
+
     #---------------------------------------------------------------------------
     # export
 
@@ -1350,7 +1428,7 @@ class IndexHierarchy(IndexBase):
         '''
         return self._INDEX_CONSTRUCTOR(self.__iter__())
 
-    def add_level(self: IH, level: tp.Hashable) -> IH:
+    def level_add(self: IH, level: tp.Hashable) -> IH:
         '''Return an IndexHierarchy with a new root (outer) level added.
         '''
         if self.STATIC: # can reuse levels
@@ -1376,11 +1454,11 @@ class IndexHierarchy(IndexBase):
 
         return self.__class__(levels, name=self._name)
 
-    def drop_level(self, count: int = 1) -> tp.Union[Index, 'IndexHierarchy']:
+    def level_drop(self, count: int = 1) -> tp.Union[Index, 'IndexHierarchy']:
         '''Return an IndexHierarchy with one or more leaf levels removed. This might change the size of the resulting index if the resulting levels are not unique.
 
         Args:
-            count: A positive value is the number of depths to remove from the root (outer) side of the hierarhcy; a negative values is the number of depths to remove from the leaf (inner) side of the hierarchy.
+            count: A positive value is the number of depths to remove from the root (outer) side of the hierarchy; a negative values is the number of depths to remove from the leaf (inner) side of the hierarchy.
         '''
         if count < 0: # remove from inner
             levels = self._levels.to_index_level()
@@ -1471,16 +1549,16 @@ class IndexHierarchyGO(IndexHierarchy):
         self._levels.extend(other._levels)
         self._recache = True
 
-    def copy(self: IH) -> IH:
+    def __copy__(self: IH) -> IH:
         '''
-        Return a new IndexHierarchy. This is not a deep copy.
+        Return a shallow copy of this IndexHierarchy.
         '''
         if self._recache:
             self._update_array_cache()
 
         blocks = self._blocks.copy()
         return self.__class__(
-                levels=self._levels.to_index_level(),
+                levels=self._levels.to_index_level(), # recursive deep copy
                 name=self._name,
                 blocks=blocks,
                 own_blocks=True,
