@@ -32,11 +32,13 @@ from static_frame.core.node_selector import InterfaceSelectDuo
 from static_frame.core.node_selector import InterfaceSelectTrio
 from static_frame.core.node_selector import TContainer
 from static_frame.core.node_str import InterfaceString
+from static_frame.core.node_transpose import InterfaceTranspose
 from static_frame.core.store import StoreConfig
 from static_frame.core.store_filter import StoreFilter
 from static_frame.core.type_blocks import TypeBlocks
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import DT64_S
+from static_frame.core.quilt import Quilt
 
 
 
@@ -283,6 +285,7 @@ class InterfaceGroup:
     OperatorUnary = 'Operator Unary'
     AccessorDatetime = 'Accessor Datetime'
     AccessorString = 'Accessor String'
+    AccessorTranspose = 'Accessor Transpose'
 
 # NOTE: order from definition retained
 INTERFACE_GROUP_ORDER = tuple(v for k, v in vars(InterfaceGroup).items() if not k.startswith('_'))
@@ -363,7 +366,7 @@ class InterfaceRecord(tp.NamedTuple):
     def gen_from_astype(cls, *,
             cls_name: str,
             name: str,
-            obj: AnyCallable,
+            obj: tp.Any,
             reference: str,
             doc: str,
             max_args: int,
@@ -523,9 +526,14 @@ class InterfaceRecord(tp.NamedTuple):
             max_args: int,
             ) -> tp.Iterator['InterfaceRecord']:
 
-        group = (InterfaceGroup.AccessorString
-                if cls_interface is InterfaceString
-                else InterfaceGroup.AccessorDatetime)
+        if cls_interface is InterfaceString:
+            group = InterfaceGroup.AccessorString
+        elif cls_interface is InterfaceDatetime:
+            group = InterfaceGroup.AccessorDatetime
+        elif cls_interface is InterfaceTranspose:
+            group = InterfaceGroup.AccessorTranspose
+        else:
+            raise NotImplementedError()
 
         for field in cls_interface.INTERFACE: # apply, map, etc
             delegate_obj = getattr(cls_interface, field)
@@ -773,6 +781,10 @@ class InterfaceSummary(Features):
             elif target is Bus:
                 f = Frame.from_elements((0,), name='frame')
                 instance = target.from_frames((f,)) #type: ignore
+            elif target is Quilt:
+                f = Frame.from_elements((0,), name='frame')
+                bus = Bus.from_frames((f,))
+                instance = target(bus, retain_labels=False) #type: ignore
             elif target is Batch:
                 instance = Batch(iter(()))
             elif issubclass(target, IndexHierarchy):
@@ -876,6 +888,11 @@ class InterfaceSummary(Features):
                             cls_interface=InterfaceDatetime,
                             **kwargs,
                             )
+            elif isinstance(obj, InterfaceTranspose):
+                yield from InterfaceRecord.gen_from_accessor(
+                            cls_interface=InterfaceTranspose,
+                            **kwargs,
+                            )
             elif obj.__class__ in (InterfaceSelectDuo, InterfaceSelectTrio):
                 yield from InterfaceRecord.gen_from_selection(
                         cls_interface=obj.__class__,
