@@ -1,12 +1,15 @@
 
 import typing as tp
-import numpy as np
+from datetime import datetime
+from datetime import date
 
+import numpy as np
 
 from static_frame.core.node_selector import Interface
 from static_frame.core.node_selector import TContainer
 from static_frame.core.util import array_from_element_attr
 from static_frame.core.util import array_from_element_method
+from static_frame.core.util import array_from_element_apply
 from static_frame.core.util import DT64_AS
 from static_frame.core.util import DT64_DAY
 from static_frame.core.util import DT64_FS
@@ -34,9 +37,6 @@ if tp.TYPE_CHECKING:
     from static_frame.core.series import Series  #pylint: disable = W0611 #pragma: no cover
     from static_frame.core.type_blocks import TypeBlocks  #pylint: disable = W0611 #pragma: no cover
 
-# only ContainerOperand subclasses
-# TContainer = tp.TypeVar('TContainer', 'Index', 'IndexHierarchy', 'Series', 'Frame', 'TypeBlocks')
-
 BlocksType = tp.Iterable[np.ndarray]
 ToContainerType = tp.Callable[[tp.Iterator[np.ndarray]], TContainer]
 
@@ -57,6 +57,8 @@ class InterfaceDatetime(Interface[TContainer]):
             'fromisoformat',
             'isoformat',
             'strftime',
+            'strptime',
+            'strpdate',
             )
 
     DT64_EXCLUDE_YEAR = (DT64_YEAR,)
@@ -285,7 +287,6 @@ class InterfaceDatetime(Interface[TContainer]):
 
         return self._blocks_to_container(blocks())
 
-
     def fromisoformat(self) -> TContainer:
         '''
         Return a :obj:`datetime.date` object from an ISO 8601 format.
@@ -307,11 +308,9 @@ class InterfaceDatetime(Interface[TContainer]):
 
         return self._blocks_to_container(blocks())
 
-
-
     def strftime(self, format: str) -> TContainer:
         '''
-        Return a string representing the date, controlled by an explicit format string.
+        Return a string representing the date, controlled by an explicit ``format`` string.
         '''
 
         def blocks() -> tp.Iterator[np.ndarray]:
@@ -336,6 +335,49 @@ class InterfaceDatetime(Interface[TContainer]):
 
         return self._blocks_to_container(blocks())
 
+    def strptime(self, format: str) -> TContainer:
+        '''
+        Return a Python datetime object from parsing a string defined with ``format``.
+        '''
+        def func(s: str) -> datetime:
+            return datetime.strptime(s, format)
+
+        def blocks() -> tp.Iterator[np.ndarray]:
+            for block in self._blocks:
+                # permit only string types, or objects types that contain strings
+                self._validate_dtype_str(block.dtype)
+                # returns an immutable array
+                array = array_from_element_apply(
+                        array=block,
+                        func=func,
+                        dtype=DTYPE_OBJECT,
+                        )
+                yield array
+
+        return self._blocks_to_container(blocks())
+
+
+    def strpdate(self, format: str) -> TContainer:
+        '''
+        Return a Python date object from parsing a string defined with ``format``.
+        '''
+        def func(s: str) -> date:
+            return datetime.strptime(s, format).date()
+
+        def blocks() -> tp.Iterator[np.ndarray]:
+            for block in self._blocks:
+                # permit only string types, or objects types that contain strings
+                self._validate_dtype_str(block.dtype)
+                # returns an immutable array
+                array = array_from_element_apply(
+                        array=block,
+                        func=func,
+                        dtype=DTYPE_OBJECT,
+                        )
+                yield array
+
+        return self._blocks_to_container(blocks())
+
 
 
 
@@ -353,8 +395,6 @@ class InterfaceDatetime(Interface[TContainer]):
 
 
 
-
-
 
 
 
