@@ -20,7 +20,16 @@ if tp.TYPE_CHECKING:
 
 #-------------------------------------------------------------------------------
 
-TContainer = tp.TypeVar('TContainer', 'Index', 'Series', 'Frame', 'TypeBlocks', 'Bus', 'Batch', 'IndexHierarchy')
+TContainer = tp.TypeVar('TContainer',
+        'Index',
+        'Series',
+        'Frame',
+        'TypeBlocks',
+        'Bus',
+        'Batch',
+        # 'Quilt',
+        'IndexHierarchy',
+        )
 GetItemFunc = tp.TypeVar('GetItemFunc',
         bound=tp.Callable[[GetItemKeyType], TContainer]
         )
