@@ -1,6 +1,7 @@
 import typing as tp
 from functools import partial
 from itertools import chain
+from copy import deepcopy
 
 import numpy as np
 from numpy.ma import MaskedArray #type: ignore
@@ -17,6 +18,7 @@ from static_frame.core.container_util import pandas_version_under_1
 from static_frame.core.container_util import rehierarch_from_index_hierarchy
 from static_frame.core.container_util import index_many_set
 from static_frame.core.container_util import index_many_concat
+from static_frame.core.container_util import sort_index_for_order
 
 from static_frame.core.display import Display
 from static_frame.core.display import DisplayActive
@@ -88,6 +90,8 @@ from static_frame.core.util import ufunc_unique
 from static_frame.core.util import write_optional_file
 from static_frame.core.util import UFunc
 from static_frame.core.util import dtype_kind_to_na
+from static_frame.core.util import DTYPE_OBJECT
+from static_frame.core.util import array_deepcopy
 
 
 if tp.TYPE_CHECKING:
@@ -109,7 +113,6 @@ class Series(ContainerOperand):
             '_name',
             )
 
-
     values: np.ndarray
 
     _index: IndexBase
@@ -140,10 +143,18 @@ class Series(ContainerOperand):
                     default_constructor=Index,
                     explicit_constructor=index_constructor
                     )
-        array = np.full(
-                len(index_final), #type: ignore
-                fill_value=element,
-                dtype=dtype)
+
+        length = len(index_final) #type: ignore
+        if isinstance(element, tuple):
+            array = np.empty(length, dtype=DTYPE_OBJECT)
+            # this is the only way to insert tuples
+            for i in range(length):
+                array[i] = element
+        else:
+            array = np.full(
+                    length,
+                    fill_value=element,
+                    dtype=dtype)
         array.flags.writeable = False
         return cls(array,
                 index=index_final,
@@ -210,7 +221,7 @@ class Series(ContainerOperand):
             containers: tp.Iterable['Series'],
             *,
             index: tp.Optional[tp.Union[IndexInitializer, IndexAutoFactoryType]] = None,
-            name: NameType = None
+            name: NameType = NAME_DEFAULT,
             ) -> 'Series':
         '''
         Concatenate multiple :obj:`Series` into a new :obj:`Series`.
@@ -226,7 +237,15 @@ class Series(ContainerOperand):
         if index is None:
             indices = []
 
+        name_first = NAME_DEFAULT
+        name_aligned = True
+
         for c in containers:
+            if name_first == NAME_DEFAULT:
+                name_first = c.name
+            elif name_first != c.name:
+                name_aligned = False
+
             array_values.append(c.values)
             if index is None:
                 indices.append(c.index)
@@ -244,6 +263,10 @@ class Series(ContainerOperand):
             # set index arg to None to force IndexAutoFactory usage in creation
             index = None
 
+        if name == NAME_DEFAULT:
+            # only derive if not explicitly set
+            name = name_first if name_aligned else None
+
         return cls(values, index=index, name=name)
 
     @classmethod
@@ -346,7 +369,7 @@ class Series(ContainerOperand):
         '''
         import pandas
         if not isinstance(value, pandas.Series):
-            raise ErrorInitSeries('from_pandas must be called with a Pandas object')
+            raise ErrorInitSeries(f'from_pandas must be called with a Pandas Series object, not: {type(value)}')
 
         if pandas_version_under_1():
             if own_data:
@@ -400,7 +423,7 @@ class Series(ContainerOperand):
         #-----------------------------------------------------------------------
         # values assignment
 
-        values_constructor = None # if deferred
+        values_constructor: tp.Optional[tp.Callable[[int], None]] = None # if deferred
 
         if not isinstance(values, np.ndarray):
             if isinstance(values, dict):
@@ -475,15 +498,6 @@ class Series(ContainerOperand):
                 f'Index has incorrect size (got {index_count}, expected {value_count})'
                 )
 
-    # ---------------------------------------------------------------------------
-    def __reversed__(self) -> tp.Iterator[tp.Hashable]:
-        '''
-        Returns a reverse iterator on the series' index.
-
-        Returns:
-            :obj:`static_frame.Series`
-        '''
-        return reversed(self._index) #type: ignore
 
     #---------------------------------------------------------------------------
     def __setstate__(self, state: tp.Any) -> None:
@@ -494,6 +508,42 @@ class Series(ContainerOperand):
             setattr(self, key, value)
         self.values.flags.writeable = False
 
+    def __deepcopy__(self, memo: tp.Dict[int, tp.Any]) -> 'Series':
+        obj = self.__new__(self.__class__)
+        obj.values = array_deepcopy(self.values, memo)
+        obj._index = deepcopy(self._index, memo)
+        obj._name = self._name # should be hashable/immutable
+
+        memo[id(self)] = obj
+        return obj #type: ignore
+
+    # def __copy__(self) -> 'Series':
+    #     '''
+    #     Return shallow copy of this Series.
+    #     '''
+    #     return self.__class__(
+    #             self._values,
+    #             index=self._index,
+    #             name=self._name,
+    #             own_index=True,
+    #             )
+
+    # def copy(self)-> 'Series':
+    #     '''
+    #     Return shallow copy of this Series.
+    #     '''
+    #     return self.__copy__() #type: ignore
+
+    # ---------------------------------------------------------------------------
+    def __reversed__(self) -> tp.Iterator[tp.Hashable]:
+        '''
+        Returns a reverse iterator on the series' index.
+
+        Returns:
+            :obj:`static_frame.Series`
+        '''
+        return reversed(self._index) #type: ignore
+
     #---------------------------------------------------------------------------
     # name interface
 
@@ -581,8 +631,6 @@ class Series(ContainerOperand):
         '''
         Interface for applying string methods to elements in this container.
         '''
-        blocks = (self.values,)
-
         def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> 'Series':
             return self.__class__(
                 next(blocks), # assume only one
@@ -591,6 +639,8 @@ class Series(ContainerOperand):
                 own_index=True,
                 )
 
+        blocks = (self.values,)
+
         return InterfaceString(
                 blocks=blocks,
                 blocks_to_container=blocks_to_container,
@@ -798,7 +848,7 @@ class Series(ContainerOperand):
 
     @doc_inject(selector='relabel', class_name='Series')
     def relabel(self,
-            index: RelabelInput
+            index: tp.Optional[RelabelInput]
             ) -> 'Series':
         '''
         {doc}
@@ -811,11 +861,11 @@ class Series(ContainerOperand):
         own_index = False
         if index is IndexAutoFactory:
             index_init = None
+        elif index is None:
+            index_init = self._index
         elif is_callable_or_mapping(index): #type: ignore
             index_init = self._index.relabel(index)
             own_index = True
-        elif index is None:
-            index_init = self._index
         else:
             index_init = index #type: ignore
 
@@ -836,8 +886,8 @@ class Series(ContainerOperand):
                 index=self._index.flat(),
                 name=self._name)
 
-    @doc_inject(selector='relabel_add_level', class_name='Series')
-    def relabel_add_level(self,
+    @doc_inject(selector='relabel_level_add', class_name='Series')
+    def relabel_level_add(self,
             level: tp.Hashable
             ) -> 'Series':
         '''
@@ -847,11 +897,11 @@ class Series(ContainerOperand):
             level: {level}
         '''
         return self.__class__(self.values,
-                index=self._index.add_level(level),
+                index=self._index.level_add(level),
                 name=self._name)
 
-    @doc_inject(selector='relabel_drop_level', class_name='Series')
-    def relabel_drop_level(self,
+    @doc_inject(selector='relabel_level_drop', class_name='Series')
+    def relabel_level_drop(self,
             count: int = 1
             ) -> 'Series':
         '''
@@ -864,7 +914,7 @@ class Series(ContainerOperand):
             raise RuntimeError('cannot drop level of an Index that is not an IndexHierarchy')
 
         return self.__class__(self.values,
-                index=self._index.drop_level(count),
+                index=self._index.level_drop(count),
                 name=self._name)
 
 
@@ -1372,11 +1422,11 @@ class Series(ContainerOperand):
         '''
         return self.values.nbytes #type: ignore
 
-    def __bool__(self) -> bool:
-        '''
-        True if this container has size.
-        '''
-        return bool(self.values.size)
+    # def __bool__(self) -> bool:
+    #     '''
+    #     True if this container has size.
+    #     '''
+    #     return bool(self.values.size)
 
 
     #---------------------------------------------------------------------------
@@ -1520,9 +1570,12 @@ class Series(ContainerOperand):
 
 
     def _axis_group_labels_items(self,
-            depth_level: DepthLevelSpecifier = 0,
+            depth_level: tp.Optional[DepthLevelSpecifier] = None,
             ) -> tp.Iterator[tp.Tuple[tp.Hashable, 'Series']]:
 
+        if depth_level is None:
+            depth_level = 0
+
         values = self.index.values_at_depth(depth_level)
         group_to_tuple = values.ndim == 2
         groups, locations = array_to_groups_and_locations(
@@ -1663,36 +1716,28 @@ class Series(ContainerOperand):
     #---------------------------------------------------------------------------
     # transformations resulting in the same dimensionality
 
+    @doc_inject(selector='sort')
     def sort_index(self,
             *,
             ascending: bool = True,
-            kind: str = DEFAULT_SORT_KIND
+            kind: str = DEFAULT_SORT_KIND,
+            key: tp.Optional[tp.Callable[[IndexBase], tp.Union[np.ndarray, IndexBase]]] = None,
             ) -> 'Series':
         '''
         Return a new Series ordered by the sorted Index.
 
         Args:
             *
-            ascending: if True, values are sorted low to high
-            kind: sort algorithm
+            ascending: {ascending}
+            kind: {kind}
+            key: {key}
 
         Returns:
-            :obj:`static_frame.Series`
+            :obj:`Series`
         '''
-        # argsort lets us do the sort once and reuse the results
-        if self._index.depth > 1:
-            v = self._index.values
-            order = np.lexsort([v[:, i] for i in range(v.shape[1]-1, -1, -1)])
-        else:
-            # this technique does not work when values is a 2d array
-            order = np.argsort(self._index.values, kind=kind)
+        order = sort_index_for_order(self._index, kind=kind, ascending=ascending, key=key)
 
-        if not ascending:
-            order = order[::-1]
-
-        index_values = self._index.values[order]
-        index_values.flags.writeable = False
-        index = self._index.from_labels(index_values, name=self._index._name)
+        index = self._index[order]
 
         values = self.values[order]
         values.flags.writeable = False
@@ -1703,25 +1748,37 @@ class Series(ContainerOperand):
                 own_index=True
                 )
 
+    @doc_inject(selector='sort')
     def sort_values(self,
             *,
             ascending: bool = True,
-            kind: str = DEFAULT_SORT_KIND
+            kind: str = DEFAULT_SORT_KIND,
+            key: tp.Optional[tp.Callable[['Series'], tp.Union[np.ndarray, 'Series']]] = None,
             ) -> 'Series':
         '''
         Return a new Series ordered by the sorted values.
 
+        Args:
+            *
+            ascending: {ascending}
+            kind: {kind}
+            key: {key}
+
         Returns:
             :obj:`Series`
         '''
+        if key:
+            cfs = key(self)
+            cfs_values = cfs if isinstance(cfs, np.ndarray) else cfs.values
+        else:
+            cfs_values = self.values
+
         # argsort lets us do the sort once and reuse the results
-        order = np.argsort(self.values, kind=kind)
+        order = np.argsort(cfs_values, kind=kind)
         if not ascending:
             order = order[::-1]
 
-        index_values = self._index.values[order]
-        index_values.flags.writeable = False
-        index = self._index.from_labels(index_values, name=self._index._name)
+        index = self._index[order]
 
         values = self.values[order]
         values.flags.writeable = False
@@ -1739,6 +1796,7 @@ class Series(ContainerOperand):
         Returns:
             :obj:`Series`
         '''
+        # returns an immutable array
         array = isin(self.values, other)
         return self.__class__(array, index=self._index, name=self._name)
 
@@ -1960,6 +2018,37 @@ class Series(ContainerOperand):
         '''
         return self.iloc[-count:]
 
+    def count(self, *,
+            skipna: bool = True
+            ) -> int:
+        '''
+        Return the count of non-NA elements.
+
+        Args:
+            skipna
+        '''
+        if not skipna:
+            return len(self.values)
+        return len(self.values) - isna_array(self.values).sum() #type: ignore
+
+    @doc_inject(selector='sample')
+    def sample(self,
+            count: int = 1,
+            *,
+            seed: tp.Optional[int] = None,
+            ) -> 'Series':
+        '''{doc}
+
+        Args:
+            {count}
+            {seed}
+        '''
+        index, key = self._index._sample_and_key(count=count, seed=seed)
+        values = self.values[key]
+        values.flags.writeable = False
+        return self.__class__(values, index=index, name=self._name)
+
+
     @doc_inject(selector='argminmax')
     def loc_min(self, *,
             skipna: bool = True
@@ -1977,7 +2066,7 @@ class Series(ContainerOperand):
         post = argmin_1d(self.values, skipna=skipna)
         if isinstance(post, FLOAT_TYPES): # NaN was returned
             raise RuntimeError('cannot produce loc representation from NaN')
-        return self.index[post]
+        return self.index[post] #type: ignore [unreachable]
 
     @doc_inject(selector='argminmax')
     def iloc_min(self, *,
@@ -2010,7 +2099,7 @@ class Series(ContainerOperand):
         post = argmax_1d(self.values, skipna=skipna)
         if isinstance(post, FLOAT_TYPES): # NaN was returned
             raise RuntimeError('cannot produce loc representation from NaN')
-        return self.index[post]
+        return self.index[post] #type: ignore [unreachable]
 
     @doc_inject(selector='argminmax')
     def iloc_max(self, *,
@@ -2158,7 +2247,7 @@ class Series(ContainerOperand):
 
         # NOTE: will only be False, or an array
         if eq is False:
-            return eq #type: ignore
+            return eq
 
         if skipna:
             isna_both = (isna_array(self.values, include_none=False) &
@@ -2256,6 +2345,17 @@ class Series(ContainerOperand):
         from static_frame import FrameGO
         return self._to_frame(constructor=FrameGO, axis=axis) #type: ignore
 
+    def to_series_he(self) -> 'SeriesHE':
+        '''
+        Return a :obj:`SeriesHE` from this :obj:`Series`.
+        '''
+        return SeriesHE(self.values,
+                index=self._index,
+                name=self._name,
+                own_index=True,
+                )
+
+
     def to_pandas(self) -> 'pandas.Series':
         '''
         Return a Pandas Series.
@@ -2355,3 +2455,55 @@ class SeriesAssign(Assign):
         return self.container.__class__(array,
                 index=self.container._index,
                 name=self.container._name)
+
+
+#-------------------------------------------------------------------------------
+class SeriesHE(Series):
+    '''
+    A hash/equals subclass of :obj:`Series`, permiting usage in a Python set, dictionary, or other contexts where a hashable container is needed. To support hashability, ``__eq__`` is implemented to return a Boolean rather than an Boolean :obj:`Series`.
+    '''
+
+    __slots__ = (
+            'values',
+            '_index',
+            '_name',
+            '_hash',
+            )
+
+    _hash: int
+
+    def __eq__(self, other: tp.Any) -> bool:
+        '''
+        Return True if other is a ``Series`` with the same labels, values, and name. Container class and underlying dtypes are not independently compared.
+        '''
+        return self.equals(other, #type: ignore
+                compare_name=True,
+                compare_dtype=False,
+                compare_class=False,
+                skipna=True,
+                )
+
+    def __ne__(self, other: tp.Any) -> bool:
+        '''
+        Return False if other is a ``Series`` with the same labels, values, and name. Container class and underlying dtypes are not independently compared.
+        '''
+        return not self.__eq__(other)
+
+
+    def __hash__(self) -> int:
+        if not hasattr(self, '_hash'):
+            self._hash = hash(tuple(self.index.values))
+        return self._hash
+
+    def to_series(self) -> Series:
+        '''
+        Return a ``Series`` from this ``SeriesHE``.
+        '''
+        return Series(self.values,
+                index=self._index,
+                name=self._name,
+                own_index=True,
+                )
+
+
+
