@@ -2,6 +2,7 @@ import unittest
 from datetime import date
 from datetime import datetime
 # from io import StringIO
+import typing as tp
 import numpy as np
 
 from static_frame.core.frame import Frame
@@ -15,8 +16,7 @@ from static_frame.core.store_zip import StoreZipTSV
 from static_frame.core.store import StoreConfigMap
 from static_frame.core.store import StoreConfig
 from static_frame.core.display_config import DisplayConfig
-
-
+from static_frame.core.hloc import HLoc
 
 from static_frame.test.test_case import TestCase
 from static_frame.test.test_case import temp_file
@@ -121,6 +121,22 @@ class TestUnit(TestCase):
             self.assertEqualFrames(f1, f1_loaded)
             self.assertEqualFrames(f2, f2_loaded)
 
+    #---------------------------------------------------------------------------
+    def test_bus_from_items_a(self) -> None:
+        f1 = Frame.from_dict(
+                dict(a=(1,2), b=(3,4)),
+                index=('x', 'y'),
+                )
+        f2 = Frame.from_dict(
+                dict(a=(1,2,3), b=(4,5,6)),
+                index=('x', 'y', 'z'),
+                )
+
+        b1 = Bus.from_items((('a', f1), ('b', f2)))
+        self.assertEqual(b1.index.values.tolist(), ['a', 'b'])
+
+
+    #---------------------------------------------------------------------------
     def test_bus_shapes_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -594,10 +610,36 @@ class TestUnit(TestCase):
 
         self.assertEqual(b2['frame'].values.tolist(),
                 [[datetime(1983, 2, 20, 5, 34, 18, 763000), datetime(2020, 8, 1, 0, 0)], [datetime(1975, 3, 20, 5, 20, 18, 1000), datetime(2020, 7, 31, 0, 0)]]
-)
+                )
 
+    def test_bus_to_xlsx_g(self) -> None:
 
+        dt64 = np.datetime64
 
+        f1 = Frame.from_dict(
+                dict(a=(1,2,3)),
+                index=('x', 'y', 'z'),
+                name=dt64('2019-12-31'))
+        f2 = Frame.from_dict(
+                dict(A=(10,20,30)),
+                index=('q', 'r', 's'),
+                name=dt64('2020-01-01'))
+
+        config = StoreConfig(include_index=True,
+                index_depth=1,
+                label_encoder=str,
+                label_decoder=dt64,
+                )
+        b1 = Bus.from_frames((f1, f2), config=config)
+
+        with temp_file('.xlsx') as fp:
+            b1.to_xlsx(fp, config=config)
+
+            b2 = Bus.from_xlsx(fp, config=config)
+            tuple(b2.items()) # force loading all
+
+        for frame in (f1, f2):
+            self.assertEqualFrames(frame, b2[frame.name])
 
     #---------------------------------------------------------------------------
     def test_bus_to_sqlite_a(self) -> None:
@@ -647,6 +689,41 @@ class TestUnit(TestCase):
         with self.assertRaises(StoreFileMutation):
             tuple(b2.items())
 
+
+    def test_bus_to_sqlite_c(self) -> None:
+
+        dt64 = np.datetime64
+
+        f1 = Frame.from_dict(
+                dict(a=(1,2,3)),
+                index=('x', 'y', 'z'),
+                name=dt64('2019-12-31'))
+        f2 = Frame.from_dict(
+                dict(A=(10,20,30)),
+                index=('q', 'r', 's'),
+                name=dt64('2020-01-01'))
+
+        config = StoreConfig(include_index=True,
+                index_depth=1,
+                label_encoder=str,
+                label_decoder=dt64,
+                )
+        b1 = Bus.from_frames((f1, f2), config=config)
+
+        with temp_file('.db') as fp:
+            b1.to_sqlite(fp, config=config)
+
+            b2 = Bus.from_sqlite(fp, config=config)
+            tuple(b2.items()) # force loading all
+            self.assertEqual(b2.index.dtype.kind, 'M')
+
+
+        for frame in (f1, f2):
+            self.assertEqualFrames(frame, b2[frame.name])
+
+
+
+    #---------------------------------------------------------------------------
     def test_bus_to_hdf5_a(self) -> None:
         f1 = Frame.from_dict(
                 dict(a=(1,2), b=(3,4)),
@@ -694,6 +771,36 @@ class TestUnit(TestCase):
         with self.assertRaises(StoreFileMutation):
             tuple(b2.items())
 
+    def test_bus_to_hdf5_c(self) -> None:
+        dt64 = np.datetime64
+
+        f1 = Frame.from_dict(
+                dict(a=(1,2,3)),
+                index=('x', 'y', 'z'),
+                name=dt64('2019-12-31'))
+        f2 = Frame.from_dict(
+                dict(A=(10,20,30)),
+                index=('q', 'r', 's'),
+                name=dt64('2020-01-01'))
+
+        config = StoreConfig(include_index=True,
+                index_depth=1,
+                label_encoder=str,
+                label_decoder=dt64,
+                )
+        b1 = Bus.from_frames((f1, f2), config=config)
+
+        with temp_file('.h5') as fp:
+            b1.to_hdf5(fp, config=config)
+
+            b2 = Bus.from_hdf5(fp, config=config)
+            tuple(b2.items()) # force loading all
+            self.assertEqual(b2.index.dtype.kind, 'M')
+
+
+        for frame in (f1, f2):
+            self.assertEqualFrames(frame, b2[frame.name])
+
     #---------------------------------------------------------------------------
 
 
@@ -763,7 +870,8 @@ class TestUnit(TestCase):
         b1 = Bus.from_frames((f1, f2, f3))
         self.assertTrue(b1.equals(b1))
 
-        class BusDerived(Bus): pass
+        class BusDerived(Bus):
+            pass
 
         b2 = BusDerived.from_frames((f1, f2, f3))
 
@@ -928,8 +1036,26 @@ class TestUnit(TestCase):
         s1 = Series((f1, f2, f3), index=ih, dtype=object)
 
         # do not support IndexHierarchy, as lables are tuples, not strings
-        with self.assertRaises(ErrorInitBus):
-            b1 = Bus(s1)
+        from ast import literal_eval
+        config = StoreConfig(label_encoder=str, label_decoder=literal_eval)
+        b1 = Bus(s1)
+        b2 = b1[HLoc[:, 1]]
+        self.assertEqual(b2.shape, (2,))
+        self.assertEqual(b2.index.values.tolist(),
+                [['a', 1], ['b', 1]])
+
+        with temp_file('.zip') as fp:
+
+            with self.assertRaises(RuntimeError):
+                b1.to_zip_pickle(fp)
+
+            b1.to_zip_pickle(fp, config=config)
+
+            # NOTE: this comes back as an Index of tuples
+            b3 = Bus.from_zip_pickle(fp, config=config)
+
+            self.assertEqual(b3.index.values.tolist(),
+                    [('a', 1), ('b', 2), ('b', 1)])
 
 
     #---------------------------------------------------------------------------
@@ -983,6 +1109,300 @@ class TestUnit(TestCase):
             self.assertEqualFrames(frame, b2[frame.name], compare_dtype=False)
 
 
+    def test_bus_to_parquet_b(self) -> None:
+
+        dt64 = np.datetime64
+
+        f1 = Frame.from_dict(
+                dict(a=(1,2), b=(3,4)),
+                index=('x', 'y'),
+                name=dt64('2020-01-01'))
+        f2 = Frame.from_dict(
+                dict(c=(1,2,3), b=(4,5,6)),
+                index=('x', 'y', 'z'),
+                name=dt64('1932-12-17'))
+        f3 = Frame.from_dict(
+                dict(d=(10,20), b=(50,60)),
+                index=('p', 'q'),
+                name=dt64('1950-04-23'))
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True,
+                label_encoder=str,
+                label_decoder=dt64
+                )
+
+        b1 = Bus.from_frames((f1, f2, f3), config=config)
+        self.assertEqual(b1.index.dtype.kind, 'M')
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_parquet(fp, config=config)
+
+            b2 = Bus.from_zip_parquet(fp, config=config)
+            self.assertEqual(b2.index.dtype.kind, 'M')
+
+            key = dt64('2020-01-01')
+            self.assertEqualFrames(b1[key], b2[key], compare_dtype=False)
+
+    #---------------------------------------------------------------------------
+    def test_bus_max_persist_a(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(20):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=3)
+            for i in b2.index:
+                _ = b2[i]
+                self.assertTrue(b2._loaded.sum() <= 3)
+
+            # after iteration only the last three are loaded
+            self.assertEqual(b2._loaded.tolist(),
+                    [False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, False, True, True, True])
+
+
+    def test_bus_max_persist_b(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(20):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+            b3 = b2.iloc[10:]
+            self.assertEqual(b3._loaded.sum(), 1)
+            # only the last one is loasded
+            self.assertEqual(b3._loaded.tolist(),
+                    [False, False, False, False, False, False, False, False, False, True]
+                    )
+            self.assertEqual(b3.iloc[0].sum().sum(), 145)
+            self.assertEqual(b3._loaded.tolist(),
+                    [True, False, False, False, False, False, False, False, False, False]
+                    )
+            self.assertEqual(b3.iloc[4].sum().sum(), 185)
+            self.assertEqual(b3._loaded.tolist(),
+                    [False, False, False, False, True, False, False, False, False, False]
+                    )
+
+    def test_bus_max_persist_c(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(4):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=4)
+
+            for _ in b2.items():
+                pass
+            self.assertTrue(b2._loaded.all())
+
+            b3 = Bus.from_zip_pickle(fp, config=config, max_persist=3)
+
+            _ = b3.iloc[[0, 2, 3]]
+            self.assertEqual(b3._loaded.tolist(),
+                    [True, False, True, True])
+
+            _ = b3.iloc[[0, 1, 3]]
+            self.assertEqual(b3._loaded.tolist(),
+                    [True, True, False, True])
+
+            _ = b3.iloc[[1, 2, 3]]
+            self.assertEqual(b3._loaded.tolist(),
+                    [False, True, True, True])
+
+            _ = b3.iloc[[0, 1, 2]]
+            self.assertEqual(b3._loaded.tolist(),
+                    [True, True, True, False])
+
+            _ = b3.iloc[[0, 2, 3]]
+            self.assertEqual(b3._loaded.tolist(),
+                    [True, False, True, True])
+
+
+
+    def test_bus_max_persist_d(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=3)
+
+            _ = b2.iloc[[0, 2, 4]]
+            self.assertEqual(b2._loaded.tolist(),
+                    [True, False, True, False, True])
+
+            _ = b2.iloc[[1, 2, 3]]
+            self.assertEqual(b2._loaded.tolist(),
+                    [False, True, True, True, False])
+
+            _ = b2.iloc[4]
+            self.assertEqual(b2._loaded.tolist(),
+                    [False, False, True, True, True])
+
+            _ = b2.iloc[0]
+            self.assertEqual(b2._loaded.tolist(),
+                    [True, False, False, True, True])
+
+            _ = b2.iloc[[2, 3, 4]]
+            self.assertEqual(b2._loaded.tolist(),
+                    [False, False, True, True, True])
+
+            _ = b2.iloc[[0, 1]]
+            self.assertEqual(b2._loaded.tolist(),
+                    [True, True, False, False, True])
+
+            _ = b2.iloc[0]
+            self.assertEqual(b2._loaded.tolist(),
+                    [True, True, False, False, True])
+
+            _ = b2.iloc[[3, 4]]
+            self.assertEqual(b2._loaded.tolist(),
+                    [True, False, False, True, True])
+
+
+    def test_bus_max_persist_e(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(4):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=4)
+
+            _ = b2.iloc[[0, 1]]
+            _ = b2.iloc[[2, 3]]
+            self.assertTrue(b2._loaded_all)
+
+            _ = b2.iloc[[1, 0]]
+            self.assertEqual(list(b2._last_accessed.keys()),
+                    ['2', '3', '1', '0'])
+
+            _ = b2.iloc[3]
+            self.assertEqual(list(b2._last_accessed.keys()),
+                    ['2', '1', '0', '3'])
+
+            _ = b2.iloc[:3]
+            self.assertEqual(list(b2._last_accessed.keys()),
+                    ['3', '0', '1', '2'])
+
+
+
+    def test_bus_max_persist_f(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+
+            # insure that items delivers Frame, not FrameDeferred
+            post = tuple(b2.items())
+            self.assertEqual(len(post), 5)
+            self.assertTrue(all(isinstance(f, Frame) for _, f in post))
+            self.assertEqual(b2._loaded.tolist(),
+                    [False, False, False, False, True])
+
+
+    def test_bus_max_persist_g(self) -> None:
+        def items() -> tp.Iterator[tp.Tuple[str, Frame]]:
+            for i in range(5):
+                yield str(i), Frame(np.arange(i, i+10).reshape(2, 5))
+
+        s = Series.from_items(items(), dtype=object)
+        b1 = Bus(s)
+
+        config = StoreConfig(
+                index_depth=1,
+                columns_depth=1,
+                include_columns=True,
+                include_index=True
+                )
+
+        with temp_file('.zip') as fp:
+            b1.to_zip_pickle(fp)
+
+            b2 = Bus.from_zip_pickle(fp, config=config, max_persist=1)
+
+            # insure that items delivers Frame, not FrameDeferred
+            post = b2.values
+            self.assertEqual(len(post), 5)
+            self.assertTrue(all(isinstance(f, Frame) for f in post))
+            self.assertEqual(b2._loaded.tolist(),
+                    [False, False, False, False, True])
+
 
 
 
