@@ -2,6 +2,8 @@ import doctest
 import os
 import typing as tp
 
+from static_frame.test.test_case import TestCase
+from static_frame.test.test_case import skip_win
 
 # useful constructors
 # >>> f = sf.FrameGO.from_records((('Encke', 3.30, '2003-12-28'), ('Giacobini-Zinner', 6.52, '1998-11-21'), ('Tempel-Tuttle', 32.92, '1998-02-28'), ('Wild 2', 6.39, '2003-09-25')), columns=('Name', 'Orbital Period', 'Perihelion Date'))
@@ -775,11 +777,11 @@ dtype('int64')
 #start_Series-interface
 >>> sf.Series.interface.loc[sf.Series.interface.index.via_str.startswith('sort')]
 <Frame: Series>
-<Index>                         cls_name group  doc                  <<U18>
+<Index>                              cls_name group  doc                  <<U18>
 <Index: signature>
-sort_index(*, ascending, kind)  Series   Method Return a new Seri...
-sort_values(*, ascending, kind) Series   Method Return a new Seri...
-<<U94>                          <<U6>    <<U17> <<U83>
+sort_index(*, ascending, kind, key)  Series   Method Return a new Seri...
+sort_values(*, ascending, kind, key) Series   Method Return a new Seri...
+<<U94>                               <<U6>    <<U17> <<U83>
 
 #end_Series-interface
 
@@ -1287,10 +1289,11 @@ d'Arrest 6.51           1.346
 <Frame: Frame>
 <Index>                              cls_name group  doc                  <<U18>
 <Index: signature>
-sort_columns(*, ascending, kind)     Frame    Method Return a new Fram...
-sort_index(*, ascending, kind)       Frame    Method Return a new Fram...
-sort_values(key, *, ascending, ax... Frame    Method Return a new Fram...
-<<U94>                               <<U5>    <<U17> <<U83>
+sort_columns(*, ascending, kind, ... Frame    Method Return a new Fram...
+sort_index(*, ascending, kind, key)  Frame    Method Return a new Fram...
+sort_values(label, *, ascending, ... Frame    Method Return a new Fram...
+<<U94>                               <<U5>    <<U18> <<U83>
+
 
 #end_Frame-interface
 
@@ -1613,7 +1616,7 @@ Mercury nan       nan
 Earth   162715536   35.640899999999995
 Mars    46131264    0.41216400000000003
 Jupiter 20444424256 3602404.0
-<<U7>   <object>    <object>
+<<U7>   <int64>     <float64>
 
 #end_Frame-iter_element().apply()
 
@@ -2218,7 +2221,7 @@ quark                              strange 0.1       -0.333
 
 #end_Frame-relabel_flat()
 
-#start_Frame-relabel_add_level()
+#start_Frame-relabel_level_add()
 >>> f = sf.Frame.from_records((('muon', 0.106, -1.0, 'lepton'), ('tau', 1.777, -1.0, 'lepton'), ('charm', 1.3, 0.666, 'quark'), ('strange', 0.1, -0.333, 'quark')), columns=('name', 'mass', 'charge', 'type'))
 >>> f = f.set_index_hierarchy(('type', 'name'), drop=True)
 >>> f
@@ -2231,7 +2234,7 @@ quark                              charm   1.3       0.666
 quark                              strange 0.1       -0.333
 <<U6>                              <<U7>   <float64> <float64>
 
->>> f.relabel_add_level(index='particle')
+>>> f.relabel_level_add(index='particle')
 <Frame>
 <Index>                                           mass      charge    <<U6>
 <IndexHierarchy: ('type', 'name')>
@@ -2241,10 +2244,10 @@ particle                           quark  charm   1.3       0.666
 particle                           quark  strange 0.1       -0.333
 <<U8>                              <<U6>  <<U7>   <float64> <float64>
 
-#end_Frame-relabel_add_level()
+#end_Frame-relabel_level_add()
 
 
-#start_Frame-relabel_drop_level()
+#start_Frame-relabel_level_drop()
 >>> f = sf.Frame.from_records((('muon', 0.106, -1.0, 'lepton'), ('tau', 1.777, -1.0, 'lepton'), ('charm', 1.3, 0.666, 'quark'), ('strange', 0.1, -0.333, 'quark')), columns=('name', 'mass', 'charge', 'type'))
 >>> f = f.set_index_hierarchy(('type', 'name'), drop=True)
 >>> f
@@ -2257,7 +2260,7 @@ quark                              charm   1.3       0.666
 quark                              strange 0.1       -0.333
 <<U6>                              <<U7>   <float64> <float64>
 
->>> f.relabel_drop_level(index=1)
+>>> f.relabel_level_drop(index=1)
 <Frame>
 <Index> mass      charge    <<U6>
 <Index>
@@ -2267,7 +2270,7 @@ charm   1.3       0.666
 strange 0.1       -0.333
 <<U7>   <float64> <float64>
 
-#end_Frame-relabel_drop_level()
+#end_Frame-relabel_level_drop()
 
 
 #start_Frame-pivot()
@@ -2403,7 +2406,7 @@ dropna(axis, condition)              FrameGO  Method   Return a new Fram...
 drop[key]                            FrameGO  Selector Label-based selec...
 drop.iloc[key]                       FrameGO  Selector
 drop.loc[key]                        FrameGO  Selector
-<<U94>                               <<U7>    <<U17>   <<U83>
+<<U94>                               <<U7>    <<U18>   <<U83>
 
 #end_FrameGO-interface
 
@@ -2548,7 +2551,7 @@ from_names(names)                    IndexHierarchy Constructor Construct a zero
 from_pandas(value)                   IndexHierarchy Constructor Given a Pandas in...
 from_product(*, name, *levels)       IndexHierarchy Constructor Given groups of i...
 from_tree(tree, *, name)             IndexHierarchy Constructor Convert into a In...
-<<U68>                               <<U14>         <<U17>      <<U83>
+<<U68>                               <<U14>         <<U18>      <<U83>
 
 #end_IndexHierarchy-interface
 
@@ -2576,7 +2579,7 @@ quark            down
 rehierarch(depth_map)     IndexHierarchyGO Method Return a new Inde...
 relabel(mapper)           IndexHierarchyGO Method Return a new Inde...
 rename(name)              IndexHierarchyGO Method Return a new Fram...
-<<U68>                    <<U16>           <<U17> <<U83>
+<<U68>                    <<U16>           <<U18> <<U83>
 
 #end_IndexHierarchyGO-interface
 
@@ -2735,9 +2738,6 @@ from_year_range(start, stop, step... IndexDateGO Constructor Get an IndexDate ..
 '''
 
 
-from static_frame.test.test_case import TestCase
-from static_frame.test.test_case import skip_win
-
 @skip_win
 class TestUnit(doctest.DocTestCase, TestCase):
 
