@@ -1,6 +1,7 @@
 import unittest
 import pickle
 import datetime
+import copy
 import typing as tp
 from io import StringIO
 
@@ -18,7 +19,7 @@ from static_frame import ILoc
 from static_frame.test.test_case import TestCase
 from static_frame.core.index import _index_initializer_needs_init
 from static_frame.core.exception import ErrorInitIndex
-from static_frame.core.index import PositionsAllocator
+from static_frame.core.util import PositionsAllocator
 from static_frame.core.util import mloc
 
 
@@ -29,6 +30,8 @@ class TestUnit(TestCase):
         a1 = PositionsAllocator.get(3)
         a2 = PositionsAllocator.get(4)
         a3 = PositionsAllocator.get(5)
+
+        # we get different object IDs, but point to the same data
         self.assertTrue(mloc(a1) == mloc(a2))
         self.assertTrue(mloc(a3) == mloc(a2))
 
@@ -483,6 +486,20 @@ class TestUnit(TestCase):
                 [index.sort(ascending=False).loc_to_iloc(x) for x in sorted(index.values)],
                 [4, 3, 2, 1, 0])
 
+
+
+    def test_index_sort_b(self) -> None:
+
+        index = Index(('ax', 'cb', 'dg', 'eb', 'bq'))
+
+        self.assertEqual(index.sort(
+                key=lambda i: i.iter_label().apply(lambda x: x[1])
+                ).values.tolist(),
+                ['cb', 'eb', 'dg', 'bq', 'ax']
+                )
+
+    #---------------------------------------------------------------------------
+
     def test_index_relable(self) -> None:
 
         index = Index(('a', 'c', 'd', 'e', 'b'))
@@ -855,8 +872,7 @@ class TestUnit(TestCase):
         self.assertEqual(list(idx1.iter_label(0)), ['a', 'b', 'c', 'd'])
 
         post = idx1.iter_label(0).apply(lambda x: x.upper())
-        self.assertEqual(post.to_pairs(),
-                ((0, 'A'), (1, 'B'), (2, 'C'), (3, 'D')))
+        self.assertEqual(post.tolist(), ['A', 'B', 'C', 'D'])
 
 
     #---------------------------------------------------------------------------
@@ -884,14 +900,20 @@ class TestUnit(TestCase):
 
     def test_index_intersection_c(self) -> None:
         idx1 = Index((10, 20))
-        idx2 = idx1.intersection(Series([20, 30]))
+        with self.assertRaises(RuntimeError):
+            # raises as it identifies labelled data
+            _ = idx1.intersection(Series([20, 30]))
+
+        idx2 = idx1.intersection(Series([20, 30]).values)
         self.assertEqual(idx2.values.tolist(), [20])
 
+        idx3 = idx1.intersection([20, 30])
+        self.assertEqual(idx3.values.tolist(), [20])
 
     def test_index_intersection_d(self) -> None:
         idx1 = Index((10, 20))
-        with self.assertRaises(NotImplementedError):
-            idx2 = idx1.intersection('b') #type: ignore
+        idx2 = idx1.intersection('b')
+        self.assertEqual(len(idx2), 0)
 
     def test_index_intersection_e(self) -> None:
 
@@ -1087,17 +1109,14 @@ class TestUnit(TestCase):
     def test_index_bool_a(self) -> None:
 
         idx1 = IndexGO(('a', 'b', 'c', 'd', 'e'))
-        self.assertTrue(bool(idx1))
-
-        idx2 = IndexGO(())
-        self.assertFalse(bool(idx2))
+        with self.assertRaises(ValueError):
+            bool(idx1)
 
     def test_index_bool_b(self) -> None:
 
         idx1 = IndexGO(())
-        self.assertFalse(bool(idx1))
-        idx1.append('a')
-        self.assertTrue(bool(idx1))
+        with self.assertRaises(ValueError):
+            bool(idx1)
 
     def test_index_astype_a(self) -> None:
 
@@ -1276,7 +1295,34 @@ class TestUnit(TestCase):
         b.append(4)
         self.assertFalse(a.equals(b))
 
+    #---------------------------------------------------------------------------
+    def test_index_sample_a(self) -> None:
+        a = IndexGO([1, 2, 3])
+        a.append(None)
+        b = a.sample(2, seed=3)
 
+        self.assertEqual(b.values.tolist(), [2, None])
+
+    #---------------------------------------------------------------------------
+    def test_index_deepcopy_a(self) -> None:
+
+        idx1 = IndexGO(('a', 'b', 'c', 'd'))
+        idx1.append('e')
+
+        idx2 = copy.deepcopy(idx1)
+        idx1.append('f')
+
+        self.assertEqual(idx2.values.tolist(), ['a', 'b', 'c', 'd', 'e'])
+        self.assertTrue(id(idx1._labels) != id(idx2._labels))
+
+    def test_index_deepcopy_b(self) -> None:
+        idx1 = IndexGO(range(5), loc_is_iloc=True)
+
+        idx2 = copy.deepcopy(idx1)
+
+        self.assertEqual(idx2.values.tolist(), [0, 1, 2, 3, 4])
+        self.assertTrue(id(idx1._labels) != id(idx2._labels))
+        self.assertTrue(idx2._map is None)
 
 if __name__ == '__main__':
     unittest.main()
