@@ -1,11 +1,10 @@
 
 import unittest
 from collections import OrderedDict
-
+import copy
 
 import numpy as np
 
-
 from static_frame import Frame
 from static_frame import HLoc
 from static_frame import ILoc
@@ -126,7 +125,7 @@ class TestUnit(TestCase):
 
     def test_index_level_values_at_depth_b(self) -> None:
 
-        hidx = IndexHierarchy.from_labels((('a', 1, 'x'), ('a', 2, 'y'), ('b', 1, 'foo'), ('b', 2, None))) #type: ignore
+        hidx = IndexHierarchy.from_labels((('a', 1, 'x'), ('a', 2, 'y'), ('b', 1, 'foo'), ('b', 2, None)))
         lvl = hidx._levels
         self.assertEqual(lvl.values_at_depth(2).tolist(), ['x', 'y', 'foo', None])
 
@@ -225,7 +224,7 @@ class TestUnit(TestCase):
 
         level0 = IndexLevel(index=Index(('a', 'b')), targets=None)
         self.assertEqual(level0.leaf_loc_to_iloc(('b',)), 1)
-        self.assertEqual(level0.leaf_loc_to_iloc(ILoc[1]), 1) #type: ignore
+        self.assertEqual(level0.leaf_loc_to_iloc(ILoc[1]), 1)
 
 
     def test_index_level_leaf_loc_to_iloc_c(self) -> None:
@@ -300,7 +299,6 @@ class TestUnit(TestCase):
         lvl2a = IndexLevelGO(index=observations)
         lvl2b = IndexLevelGO(index=observations, offset=2)
         lvl2_targets = ArrayGO((lvl2a, lvl2b))
-        # must defensively copy index
         assert id(lvl2a.index) != id(lvl2b.index)
 
         lvl1a = IndexLevelGO(index=groups,
@@ -402,13 +400,13 @@ class TestUnit(TestCase):
 
         levels = IndexLevel.from_tree(tree)
 
-        post = list(levels.label_nodes_at_depth(0))
+        post = list(levels.index_at_depth(0))
         self.assertEqual(post, ['I', 'II'])
 
-        post = list(levels.label_nodes_at_depth(1))
+        post = list(levels.index_at_depth(1))
         self.assertEqual(post, ['A', 'B', 'C', 'A', 'B'])
 
-        post = list(levels.label_nodes_at_depth(2))
+        post = list(levels.index_at_depth(2))
         self.assertEqual(post, [1, 2, 1, 2, 3, 2, 3, 1, 2, 3, 1])
 
 
@@ -674,7 +672,7 @@ class TestUnit(TestCase):
         self.assertEqual(tuple(lvl1.dtype_per_depth()),
                 (dtype('float64'), dtype('float64'), dtype('float64')))
 
-        self.assertEqual(tuple(lvl1.labels_at_depth(0)), (EMPTY_ARRAY,))
+        self.assertEqual(tuple(lvl1.index_array_at_depth(0)), (EMPTY_ARRAY,))
 
         self.assertEqual(lvl1.values_at_depth(0).tolist(), EMPTY_ARRAY.tolist())
 
@@ -688,10 +686,55 @@ class TestUnit(TestCase):
         self.assertEqual(lvl1.depth, 4)
         self.assertEqual(len(lvl1), 0)
 
+    #---------------------------------------------------------------------------
+    def test_index_level_labels_at_depth_a(self) -> None:
+
+        OD = OrderedDict
+        tree = OD([
+                ('I', OD([
+                        ('A', (1, 2)), ('B', (1, 2, 3)), ('C', (2, 3))
+                        ])
+                ),
+                ('II', OD([
+                        ('A', (1,)), ('B', (1,))
+                        ])
+                ),
+                ('III', OD([
+                        ('A', (1, 2, 3)), ('B', (1,))
+                        ])
+                ),
+                ])
+        levels = IndexLevel.from_tree(tree)
+
+        self.assertEqual(tuple(levels.labels_at_depth(0)),
+                ('I', 'I', 'I', 'I', 'I', 'I', 'I', 'II', 'II', 'III', 'III', 'III', 'III'))
+        self.assertEqual(tuple(levels.labels_at_depth(1)),
+                ('A', 'A', 'B', 'B', 'B', 'C', 'C', 'A', 'B', 'A', 'A', 'A', 'B'))
+        self.assertEqual(tuple(levels.labels_at_depth(2)),
+                (1, 2, 1, 2, 3, 2, 3, 1, 1, 1, 2, 3, 1))
 
 
+    #---------------------------------------------------------------------------
 
+    def test_index_level_deepcopy_a(self) -> None:
 
+        groups = IndexGO(('A', 'B'))
+        observations = IndexGO(('x', 'y'))
+
+        lvl2a = IndexLevelGO(index=observations)
+        lvl2b = IndexLevelGO(index=observations, offset=2)
+        lvl2_targets = ArrayGO((lvl2a, lvl2b))
+        assert id(lvl2a.index) != id(lvl2b.index)
+
+        lvl1a = IndexLevelGO(index=groups,
+                targets=lvl2_targets,
+                offset=0)
+
+        post = copy.deepcopy(lvl1a)
+        self.assertEqual(lvl1a.values.tolist(), post.values.tolist())
+        self.assertEqual(
+                id(post.targets[0].index._labels), #type: ignore
+                id(post.targets[1].index._labels)) #type: ignore
 
 
 if __name__ == '__main__':
